@@ -25,6 +25,7 @@
 #include "gtk24backports.h"
 #include "kz-entry-action.h"
 #include "gtk-utils.h"
+#include "glib-utils.h"
 #include "intl.h"
 #include "kazehakase.h"
 #include "kz-window.h"
@@ -33,7 +34,6 @@
 #include "mozilla-prefs.h"
 #include "kz-mozembed.h"
 #include "kz-favicon.h"
-#include "glib-utils.h"
 
 #define KZ_ACTIONS_POPUP_LANGUAGE_KEY "KzActionsPopup::Language"
 #define KZ_ACTIONS_POPUP_TAB_KEY "KzActionsPopup::Tab"
@@ -213,6 +213,23 @@ act_popup_open_frame (GtkAction *action, KzWindow *kz)
 	open_location(action, kz, LOCATION_FRAME, CURRENT_TAB);
 }
 
+static void
+cb_embed_changed (GtkMozEmbed *embed, EditorInfo *einfo)
+{
+	g_signal_handlers_disconnect_by_func(G_OBJECT(embed),
+			 		     G_CALLBACK(cb_embed_changed),
+					     einfo);
+	if (!einfo) return;
+
+	if (einfo->filename)
+		g_free(einfo->filename);
+	einfo->filename = NULL;
+	if (einfo->kzembed)
+		g_object_unref(einfo->kzembed);
+	einfo->kzembed  = NULL;
+	einfo->element  = NULL;
+}
+
 static void
 cb_editor_exit (GPid pid, gint status, gpointer data)
 {
@@ -220,36 +237,50 @@ cb_editor_exit (GPid pid, gint status, gpointer data)
 	gsize size;
 	EditorInfo *einfo;
 	
-	GIOChannel *io;
-	
 	/* close editor process */
 #if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION > 3)
 	g_spawn_close_pid(pid);
 #endif
-	if (!data)
-		return;
-	einfo = (EditorInfo*)data;
+	if (!data) return;
 
-	/* get text from temporary file */
-	io = g_io_channel_new_file(einfo->filename, "r", NULL);
-	g_io_channel_read_to_end(io, &text, &size, NULL);
-	g_io_channel_unref(io);
-	
-	/* set text into textarea */
-	if (text)
+	einfo = (EditorInfo*)data;
+	if (einfo->kzembed)
 	{
-		kz_moz_embed_set_text_into_textarea(einfo->kzembed,
-						    einfo->element,
-						    text);
-		g_free(text);
+		g_signal_handlers_disconnect_by_func(G_OBJECT(einfo->kzembed),
+			 		     G_CALLBACK(cb_embed_changed),
+					     einfo);
 	}
 
-	g_free(einfo->filename);
-	einfo->filename = NULL;
+	if (einfo->filename)
+	{
+		/* get text from temporary file */
+		gboolean ret;
+		ret = g_file_get_contents(einfo->filename, &text, &size, NULL);
+		/* set text into textarea */
+		if (ret)
+		{
+			if (KZ_IS_MOZ_EMBED(einfo->kzembed) && einfo->element)
+			{
+				kz_moz_embed_set_text_into_textarea(einfo->kzembed,
+								    einfo->element,
+								    text);
+			}
+			g_free(text);
+		}
+
+		g_free(einfo->filename);
+		einfo->filename = NULL;
+	}
+	
+	if (einfo->kzembed)
+	{
+		g_object_unref(einfo->kzembed);
+	}
 	einfo->kzembed  = NULL;
 	einfo->element  = NULL;
 	
 	g_free(einfo);
+	einfo = NULL;
 }
 
 
@@ -280,7 +311,7 @@ act_popup_launch_editor (GtkAction *action, KzWindow *kz)
 	if (event->cinfo.context & KZ_CONTEXT_TEXTAREA)
 	{
 		einfo = g_new0(EditorInfo, 1);
-		einfo->kzembed  = KZ_MOZ_EMBED(widget);
+		einfo->kzembed  = g_object_ref(KZ_MOZ_EMBED(widget));
 		einfo->element  = event->cinfo.element;
 		text = kz_moz_embed_get_text_from_textarea(KZ_MOZ_EMBED(widget),
 							   einfo->element);
@@ -301,7 +332,12 @@ act_popup_launch_editor (GtkAction *action, KzWindow *kz)
 	{
 		command = g_strdup_printf(editor_command, "");
 	}
-
+	
+	/* */
+	g_signal_connect(G_OBJECT(widget), "net_start",
+			 G_CALLBACK(cb_embed_changed), einfo);
+	g_signal_connect(G_OBJECT(widget), "destroy",
+			 G_CALLBACK(cb_embed_changed), einfo);
 	/* launch editor */
 	g_shell_parse_argv(command,
 			   &argc,
@@ -383,7 +419,6 @@ act_popup_tab_list (GtkAction *action, KzWindow *kz)
 					     G_CALLBACK(cb_popup_menu_hide), kz);
 }
 
-
 GtkActionEntry kz_actions_popup[] =
 {
         /* Toplevel */
