@@ -18,13 +18,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-edit.c,v 1.18 2004/07/23 07:36:49 ikezoe Exp $
+ *  $Id: kz-bookmark-edit.c,v 1.24 2004/08/13 07:23:44 ikezoe Exp $
  */
 
 #include "kz-bookmark-edit.h"
 #include "gobject-utils.h"
 #include "gtk-utils.h"
 #include "intl.h"
+#include "kz-thumbnail.h"
 
 #define REGEX_ENTRY_WIDTH 100
 #define URI_ENTRY_WIDTH 150
@@ -108,13 +109,14 @@ kz_bookmark_edit_init (KzBookmarkEdit *edit)
 	GtkWidget *location_label, *location_entry;
 	GtkWidget *interval_label, *interval_spin;
 	GtkWidget *memo_label, *textview;
+	GtkWidget *thumbnail;
 	GtkTextBuffer *textbuf;
 
 	edit->smart = g_new0(KzSmartBookmarkEdit, 1);
 	
 	main_vbox = GTK_VBOX(edit);
 
-	table = gtk_table_new(5, 2, FALSE);
+	table = gtk_table_new(6, 3, FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
 	gtk_box_pack_start(GTK_BOX(main_vbox), table, TRUE, TRUE, 0);
 	gtk_widget_show(table);
@@ -203,7 +205,7 @@ kz_bookmark_edit_init (KzBookmarkEdit *edit)
 	/* Memo */
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_table_attach(GTK_TABLE(table), hbox,
-			 0, 1, 4, 5,  /* xpos, ypos */
+			 0, 3, 4, 5,  /* xpos, ypos */
 			 GTK_FILL, 0,
 			 2, 2);       /* padding x, y  */
 	gtk_widget_show(hbox);
@@ -219,7 +221,7 @@ kz_bookmark_edit_init (KzBookmarkEdit *edit)
         gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwin),
 					    GTK_SHADOW_IN);
 	gtk_table_attach(GTK_TABLE(table), scrwin,
-			 1, 2, 4, 5,  /* xpos, ypos */
+			 1, 3, 4, 5,  /* xpos, ypos */
 			 GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
 			 2, 2);       /* padding x, y  */
 	gtk_widget_show(scrwin);
@@ -235,10 +237,18 @@ kz_bookmark_edit_init (KzBookmarkEdit *edit)
 	/* properties for smart bookmark */
 	edit->smart->box = create_widget_for_smart_property(edit);
 	gtk_table_attach(GTK_TABLE(table), edit->smart->box,
-			 0, 2, 5, 6,  /* xpos, ypos */
+			 0, 3, 5, 6,  /* xpos, ypos */
 			 GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
 			 2, 2);       /* padding x, y  */
 
+	/* thumbnail */
+	thumbnail = kz_thumbnail_new();
+	gtk_table_attach(GTK_TABLE(table), thumbnail,
+			 2, 3, 0, 4,
+			 GTK_SHRINK, GTK_SHRINK,
+			 2, 2);
+	gtk_widget_show(thumbnail);
+
 	/* initialize */
 	edit->title_label    = title_label;
 	edit->title_entry    = title_entry;
@@ -252,6 +262,7 @@ kz_bookmark_edit_init (KzBookmarkEdit *edit)
 	edit->memo_textview  = textview;
 	edit->memo_scrwin    = scrwin;
 	edit->bookmark       = NULL;
+	edit->thumbnail      = thumbnail;
 	edit->changing       = FALSE;
 }
 
@@ -467,7 +478,13 @@ kz_bookmark_edit_sync_properties (KzBookmarkEdit *edit, KzBookmark *bookmark)
 	if (title)
 		gtk_entry_set_text(GTK_ENTRY(edit->title_entry), title);
 	if (uri)
+	{
 		gtk_entry_set_text(GTK_ENTRY(edit->uri_entry), uri);
+		
+		kz_thumbnail_set_thumbnail_from_uri(
+				KZ_THUMBNAIL(edit->thumbnail),
+				uri);
+	}
 	if (location)
 		gtk_entry_set_text(GTK_ENTRY(edit->location_entry), location);
 	adj = gtk_spin_button_get_adjustment
