@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-editor.c,v 1.34 2004/03/25 04:32:23 makeinu Exp $
+ *  $Id: kz-bookmark-editor.c,v 1.36 2004/07/23 03:20:55 ikezoe Exp $
  */
 
 #include "kz-bookmark-editor.h"
@@ -27,7 +27,7 @@
 #include "kazehakase.h"
 #include "gobject-utils.h"
 #include "intl.h"
-#include "egg-toggle-action.h"
+#include "gtk24backports.h"
 #include "kz-window.h"
 #include "kz-bookmark-edit.h"
 #include "kz-actions-bookmark.h"
@@ -63,7 +63,7 @@ static void     kz_bookmark_editor_set_folder_tree   (KzBookmarkEditor *editor);
 static void     kz_bookmark_editor_set_bookmark_tree (KzBookmarkEditor *editor,
 						      KzBookmark       *parent);
 
-static void     cb_menu_merge_add_widget        (EggMenuMerge     *merge,
+static void     cb_menu_merge_add_widget        (GtkUIManager     *merge,
 						 GtkWidget        *widget,
 						 GtkBox           *box);
 static void     cb_folder_view_cursor_changed   (GtkTreeView      *treeview,
@@ -119,10 +119,10 @@ kz_bookmark_editor_class_init (KzBookmarkEditorClass *klass)
 
 #define SET_SENSITIVE(editor, name, sensitive)				\
 {									\
-	EggAction *action;						\
-	action = egg_action_group_get_action((editor)->action_group,	\
+	GtkAction *action;						\
+	action = gtk_action_group_get_action((editor)->action_group,	\
 					     (name));			\
-	egg_action_set_sensitive(action, (sensitive));			\
+	g_object_set(action, "sensitive", (sensitive), NULL);			\
 }
 
 
@@ -132,9 +132,9 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 	GtkWidget *hpaned, *vpaned, *main_vbox;
 	GtkWidget *scrwin[3], *treeview[2], *edit_view, *statusbar;
 	GtkTreeSelection *selection;
-	GtkAccelGroup  *accels;
-	EggActionGroup *actions;
-	EggMenuMerge   *menu_merge;
+	GtkActionGroup *actions;
+	GtkAccelGroup *accel_group;
+	GtkUIManager   *menu_merge;
 	GError *err = NULL;
 
 	g_object_set(G_OBJECT(editor), "allow-shrink", TRUE, NULL);
@@ -144,11 +144,10 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 	gtk_window_set_position(GTK_WINDOW(editor),
 				GTK_WIN_POS_CENTER_ON_PARENT);
 
-	accels     = gtk_accel_group_new();
-	actions    = kz_actions_bookmark_create_group(editor, accels);
-	menu_merge = egg_menu_merge_new();
+	menu_merge = gtk_ui_manager_new();
+	accel_group = gtk_ui_manager_get_accel_group(menu_merge);
 
-	gtk_window_add_accel_group(GTK_WINDOW(editor), accels);
+	actions    = kz_actions_bookmark_create_group(editor, accel_group);
 
 	/* toplevel vbox */
 	main_vbox = gtk_vbox_new(FALSE, 0);
@@ -156,11 +155,10 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 	gtk_widget_show(main_vbox);
 
 	/* menu and toolbar */
-	egg_menu_merge_set_accel_group(menu_merge, accels);
-	egg_menu_merge_insert_action_group(menu_merge, actions, 0);
+	gtk_ui_manager_insert_action_group(menu_merge, actions, 0);
 	g_signal_connect(menu_merge, "add_widget",
 			 G_CALLBACK(cb_menu_merge_add_widget), main_vbox);
-	egg_menu_merge_add_ui_from_file(menu_merge,
+	gtk_ui_manager_add_ui_from_file(menu_merge,
 					KZ_SYSCONFDIR"/kz-ui-bookmarks.xml",
 					&err);
 	if (err)
@@ -168,8 +166,11 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 		g_warning("%s", err->message);
 		g_error_free(err);
 	}
-        egg_menu_merge_ensure_update(menu_merge);
+        gtk_ui_manager_ensure_update(menu_merge);
 
+	gtk_window_add_accel_group(GTK_WINDOW(editor), 
+				   gtk_ui_manager_get_accel_group(menu_merge));
+	
 	/* paned for main contents */
 	hpaned = gtk_hpaned_new();
 	gtk_box_pack_start(GTK_BOX(main_vbox), hpaned, TRUE, TRUE, 0);
@@ -396,6 +397,7 @@ kz_bookmark_editor_set_bookmark_tree (KzBookmarkEditor *editor,
 	SET_SENSITIVE(editor, "InsertBookmark",       editable);
 	SET_SENSITIVE(editor, "InsertFolder",         editable);
 	SET_SENSITIVE(editor, "InsertRemoteBookmark", editable);
+	SET_SENSITIVE(editor, "InsertSmartBookmark",  editable);
 	SET_SENSITIVE(editor, "InsertSeparator",      editable);
 }
 
@@ -438,7 +440,7 @@ kz_bookmark_editor_set_tree_mode (KzBookmarkEditor *editor,
 void
 kz_bookmark_editor_store_state (KzBookmarkEditor *editor)
 {
-	EggAction *action;
+	GtkAction *action;
 	gint client_x, client_y, width, height;
 	gboolean show_folder_view, show_edit_view;
 
@@ -448,15 +450,15 @@ kz_bookmark_editor_store_state (KzBookmarkEditor *editor)
 				 &client_x, &client_y,
 				 &width, &height, NULL);
 
-	action = egg_action_group_get_action(editor->action_group,
+	action = gtk_action_group_get_action(editor->action_group,
 					     "ShowHideFolderView");
 	show_folder_view
-		= egg_toggle_action_get_active(EGG_TOGGLE_ACTION(action));
+		= gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
 
-	action = egg_action_group_get_action(editor->action_group,
+	action = gtk_action_group_get_action(editor->action_group,
 					     "ShowHideContentView");
 	show_edit_view
-		= egg_toggle_action_get_active(EGG_TOGGLE_ACTION(action));
+		= gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
 
 	KZ_CONF_SET("BookmarkEditorWindow", "width",  width,  INT);
 	KZ_CONF_SET("BookmarkEditorWindow", "height", height, INT);
@@ -479,9 +481,9 @@ kz_bookmark_editor_store_state (KzBookmarkEditor *editor)
 	}
 
 	/* mode */
-	action = egg_action_group_get_action(editor->action_group,
+	action = gtk_action_group_get_action(editor->action_group,
 					     "TreeMode");
-	if (egg_toggle_action_get_active(EGG_TOGGLE_ACTION(action)))
+	if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)))
 		KZ_CONF_SET_STR("BookmarkEditorWindow", "mode",
 				"TreeMode");
 	else
@@ -493,7 +495,7 @@ kz_bookmark_editor_store_state (KzBookmarkEditor *editor)
 void
 kz_bookmark_editor_restore_state (KzBookmarkEditor *editor)
 {
-	EggAction *action;
+	GtkAction *action;
 	gint width = 600, height = 450;
 	gint folder_view_width = 150, bookmarks_view_height = 230;
 	gboolean show_folder_view = TRUE, show_edit_view = FALSE;
@@ -516,14 +518,14 @@ kz_bookmark_editor_restore_state (KzBookmarkEditor *editor)
 	/* size */
 	gtk_window_set_default_size(GTK_WINDOW(editor), width, height);
 
-	action = egg_action_group_get_action(editor->action_group,
+	action = gtk_action_group_get_action(editor->action_group,
 					     "ShowHideFolderView");
-	egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action),
+	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action),
 				     show_folder_view);
 
-	action = egg_action_group_get_action(editor->action_group,
+	action = gtk_action_group_get_action(editor->action_group,
 					     "ShowHideContentView");
-	egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action),
+	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action),
 				     show_edit_view);
 
 	gtk_widget_set_size_request(GTK_WIDGET(editor->scrolledwin[0]),
@@ -534,15 +536,15 @@ kz_bookmark_editor_restore_state (KzBookmarkEditor *editor)
 	/* mode */
 	if (mode && !strcmp(mode, "TreeMode"))
 	{
-		action = egg_action_group_get_action(editor->action_group,
+		action = gtk_action_group_get_action(editor->action_group,
 						     "TreeMode");
-		egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action), TRUE);
+		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), TRUE);
 	}
 	else
 	{
-		action = egg_action_group_get_action(editor->action_group,
+		action = gtk_action_group_get_action(editor->action_group,
 						     "ListMode");
-		egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action), TRUE);
+		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), TRUE);
 	}
 
 	g_free(mode);
@@ -550,7 +552,7 @@ kz_bookmark_editor_restore_state (KzBookmarkEditor *editor)
 
 
 static void
-cb_menu_merge_add_widget (EggMenuMerge *merge, GtkWidget *widget,
+cb_menu_merge_add_widget (GtkUIManager *merge, GtkWidget *widget,
 			  GtkBox *box)
 {
 	gtk_box_pack_start (box, widget, FALSE, FALSE, 0);
