@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark.c,v 1.45 2004/07/18 09:10:13 ikezoe Exp $
+ *  $Id: kz-bookmark.c,v 1.55 2004/09/24 13:58:48 ikezoe Exp $
  */
 
 #include "kz-bookmark.h"
@@ -26,6 +26,7 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include "gobject-utils.h"
+#include "glib-utils.h"
 #include "intl.h"
 #include "kz-marshalers.h"
 #include "utils.h"
@@ -39,7 +40,6 @@
 #include "kz-nsbookmark.h"
 #include "kz-lirs.h"
 #include "kz-hinadi.h"
-#include "kz-scrap-bookmark.h"
 
 enum {
 	INSERT_CHILD_SIGNAL,
@@ -64,6 +64,8 @@ enum {
 	PROP_FILE_TYPE,
 	PROP_DESCRIPTION,
 	PROP_LAST_MODIFIED,
+	PROP_LAST_VISITED,
+	PROP_ADDED_TIME,
 	PROP_INTERVAL,
 	PROP_SMART_LIST,
 	PROP_EDITABLE
@@ -125,6 +127,8 @@ static GQuark location_quark          = 0;
 static GQuark file_type_quark         = 0;
 static GQuark description_quark       = 0;
 static GQuark last_mod_quark          = 0;
+static GQuark last_visited_quark      = 0;
+static GQuark added_time_quark        = 0;
 static GQuark smart_list_quark        = 0;
 static GQuark parent_quark            = 0;
 static GQuark children_quark          = 0;
@@ -171,7 +175,6 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 		ADD_BOOKMARK_FILE_TYPE(kz_hinadi_get_file_types);
 		ADD_BOOKMARK_FILE_TYPE(kz_xbel_get_file_types);
 		ADD_BOOKMARK_FILE_TYPE(kz_nsbookmark_get_file_types);
-		ADD_BOOKMARK_FILE_TYPE(kz_scrap_bookmark_get_file_types);
 	}
 
 	parent_class = g_type_class_peek_parent (klass);
@@ -273,6 +276,30 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 			 0,
 			 G_PARAM_READWRITE));
 
+	g_object_class_install_property(
+		object_class,
+		 PROP_LAST_VISITED,
+		 g_param_spec_uint(
+			 "last-visited",
+			 _("Last Visited Time"),
+			 _("The time of the user's last visit to the link"),
+			 0,
+			 G_MAXUINT,
+			 0,
+			 G_PARAM_READWRITE));
+
+	g_object_class_install_property(
+		object_class,
+		 PROP_ADDED_TIME,
+		 g_param_spec_uint(
+			 "added-time",
+			 _("Added Time"),
+			 _("The time of the added the bookmark"),
+			 0,
+			 G_MAXUINT,
+			 0,
+			 G_PARAM_READWRITE));
+
 	g_object_class_install_property(
 		object_class,
 		 PROP_INTERVAL,
@@ -411,6 +438,8 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 	file_type_quark   = g_quark_from_string("KzBookmark::FileType");
 	description_quark = g_quark_from_string("KzBookmark::Description");
 	last_mod_quark    = g_quark_from_string("KzBookmark::LastModified");
+	last_visited_quark= g_quark_from_string("KzBookmark::LastVisited");
+	added_time_quark  = g_quark_from_string("KzBookmark::AddedTime");
 	parent_quark      = g_quark_from_string("KzBookmark::Parent");
 	children_quark    = g_quark_from_string("KzBookmark::Children");
 	interval_quark    = g_quark_from_string("KzBookmark::Interval");
@@ -582,6 +611,16 @@ kz_bookmark_set_property (GObject *object,
 		g_object_set_qdata(object, last_mod_quark,
 				   GUINT_TO_POINTER(g_value_get_uint(value)));
 		break;
+	case PROP_LAST_VISITED:
+		g_return_if_fail(!kz_bookmark_is_separator(bookmark));
+		g_object_set_qdata(object, last_visited_quark,
+				   GUINT_TO_POINTER(g_value_get_uint(value)));
+		break;
+	case PROP_ADDED_TIME:
+		g_return_if_fail(!kz_bookmark_is_separator(bookmark));
+		g_object_set_qdata(object, added_time_quark,
+				   GUINT_TO_POINTER(g_value_get_uint(value)));
+		break;
 	case PROP_INTERVAL:
 	{
 		guint interval, timer_id;
@@ -667,6 +706,16 @@ kz_bookmark_get_property (GObject *object,
 							   last_mod_quark));
 		g_value_set_uint(value, time);
 		break;
+	case PROP_LAST_VISITED:
+		time = GPOINTER_TO_UINT(g_object_get_qdata(object,
+							   last_visited_quark));
+		g_value_set_uint(value, time);
+		break;
+	case PROP_ADDED_TIME:
+		time = GPOINTER_TO_UINT(g_object_get_qdata(object,
+							   added_time_quark));
+		g_value_set_uint(value, time);
+		break;
 	case PROP_INTERVAL:
 	{
 		guint interval;
@@ -856,6 +905,22 @@ kz_bookmark_get_last_modified (KzBookmark *bookmark)
 						   last_mod_quark));
 }
 
+guint
+kz_bookmark_get_last_visited (KzBookmark *bookmark)
+{
+	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), 0);
+	return GPOINTER_TO_UINT(g_object_get_qdata(G_OBJECT(bookmark),
+						   last_visited_quark));
+}
+
+guint
+kz_bookmark_get_added_time (KzBookmark *bookmark)
+{
+	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), 0);
+	return GPOINTER_TO_UINT(g_object_get_qdata(G_OBJECT(bookmark),
+						   added_time_quark));
+}
+
 
 guint
 kz_bookmark_get_interval (KzBookmark *bookmark)
@@ -974,6 +1039,22 @@ kz_bookmark_set_last_modified (KzBookmark *bookmark,
 	g_object_set(G_OBJECT(bookmark), "last-modified", time, NULL);
 }
 
+void
+kz_bookmark_set_last_visited (KzBookmark *bookmark,
+			      guint time)
+{
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+	g_object_set(G_OBJECT(bookmark), "last-visited", time, NULL);
+}
+
+void
+kz_bookmark_set_added_time (KzBookmark *bookmark,
+		            guint time)
+{
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+	g_object_set(G_OBJECT(bookmark), "added-time", time, NULL);
+}
+
 
 void
 kz_bookmark_set_interval (KzBookmark *bookmark, guint interval)
@@ -1099,25 +1180,30 @@ kz_bookmark_remove_smart_property (KzBookmark *bookmark)
 {
 }
 
-#define MAX_GROUPSTRING 16
+#define MAX_GROUPSTRING 9
 gchar *
 kz_bookmark_get_smart_uri (KzBookmark *bookmark, const gchar *text)
 {
 	gchar *smart_uri;
 	GList *smart_list, *node;
 	KzBookmarkSmartProperty *match = NULL;
+	gint n_match = 0;
 	EggRegex *egg_regex;
 	GError *error = NULL;
 
 	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);
 
-	if (!(bookmark->flags & KZ_BOOKMARK_SMART_FLAG))
-		return NULL;
+	if (!text) 
+		return g_strdup("");
 
-	if (!text)
+	if (!(bookmark->flags & KZ_BOOKMARK_SMART_FLAG))
 	{
-		smart_uri = g_strdup(kz_bookmark_get_link(bookmark));
-		return smart_uri;
+		const gchar *link;
+		link = kz_bookmark_get_link(bookmark);
+		if (link)
+			return g_strdup(link);
+		else 
+			return g_strdup(text);
 	}
 
 	smart_list = g_object_get_qdata(G_OBJECT(bookmark),
@@ -1137,10 +1223,12 @@ kz_bookmark_get_smart_uri (KzBookmark *bookmark, const gchar *text)
 		{
 			g_warning("KzBookmark: Regular Expression Error");
 			g_error_free(error);
-			return NULL;
+			return g_strdup(text);
 		}
 
-		if (egg_regex_match(egg_regex, text, -1, 0) > 0)
+		n_match = egg_regex_match(egg_regex, text, -1, 0);
+
+		if (n_match > 0)		
 		{
 			match = prop;
 			break;
@@ -1164,94 +1252,42 @@ kz_bookmark_get_smart_uri (KzBookmark *bookmark, const gchar *text)
 		{
 			while (strings[n] && n < MAX_GROUPSTRING)
 			{
-				GError *e = NULL;
-				gchar *encode_string = NULL;
-
-				if (encode)
-				{
-					encode_string = 
-						g_convert(strings[n],
-							  strlen(strings[n]),
-							  encode, "UTF-8",
-							  NULL, NULL,
-							  &e);
-				}
-				
-				if (e || !encode)
-				{
-					encode_string =
-					    g_strdup(strings[n]);
-					if (e)
-						g_error_free(e);
-				}
-				if (urlencode)	
-				{
-					url_strings[n] = 
-						url_encode(encode_string);
-				}
-				else
-				{
-					url_strings[n] = g_strdup(encode_string);
-				}
-
-				g_free(encode_string);
+				url_strings[n] = ensure_encode_string(strings[n], 
+						     encode,
+						     urlencode);
 				n++;
 			}
 		}
 		g_strfreev(strings);	
 
+		/* %s is placed whole of the text */
 		str = g_string_new_len(uri, strlen(uri));
 		if (strstr(uri, "%s"))
 		{
-			gchar *encode_string = NULL;
 			gchar *url;
-			GError *e = NULL;
-
-			if (encode)
-			{
-				encode_string = 
-					g_convert(text,
-						  strlen(text),
-						  encode, "UTF-8",
-						  NULL, NULL,
-						  &e);
-			}
-				
-			if (e || !encode)
-			{
-				encode_string =
-				    g_strdup(text);
-				if (e)
-					g_error_free(e);
-			}
-			if (urlencode)	
-			{
-				url = url_encode(encode_string);
-			}
-			else
-			{
-				url = g_strdup(encode_string);
-			}
+			url = ensure_encode_string(text, encode,
+						   urlencode);
 			g_string_printf(str, uri, url);
 			g_free(url);
-			g_free(encode_string);
 		}
 
+		/* \1, \2 ... is replaced matching sub pattern */
 		for (i = 0; i < n; i++)
 		{
 			gchar *pos;
 			gchar *index;
 			index = g_strdup_printf("\\%d", i);
 			pos = strstr(str->str, index); 
-			if (pos)
+			while (pos)
 			{
 				gssize p = pos - str->str;
 				str = g_string_erase(str, p, strlen(index));
 				str = g_string_insert(str, p, url_strings[i]); 
+				pos = strstr(str->str, index);
 			}
 			g_free(index);
 		}
-		
+
 		smart_uri = g_strndup(str->str, str->len);	
 		g_string_free(str, TRUE);
 
@@ -1527,12 +1563,18 @@ kz_bookmark_load_start (KzBookmark *bookmark)
 	KzIO *io;
 	const gchar *uri;
 	guint id;
+	KzBookmarkState state;
 
 	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), FALSE);
 
 	uri = kz_bookmark_get_location(KZ_BOOKMARK(bookmark));
 	if (!uri) return FALSE;
 
+	/* preventing to insert children bookmark twice, but silly! */
+	state = kz_bookmark_get_state(bookmark);
+	if (state == KZ_BOOKMARK_STATE_LOADING)
+		return TRUE;
+	
 	/* check loading state */
 	if (kz_bookmark_is_loading_all_children(bookmark))
 		return TRUE;
@@ -1807,3 +1849,38 @@ kz_bookmark_is_loading_all_children (KzBookmark *bookmark)
 }
 
 
+KzBookmark *
+kz_bookmark_find_bookmark_from_uri (KzBookmark *folder, const gchar *key_uri)
+{
+	GList *children, *node;
+	KzBookmark *bookmark = NULL;
+
+	children = kz_bookmark_get_children(folder);
+
+	/* FIXME! Use hash? */
+	for (node = children; node; node = g_list_next(node))
+	{
+		KzBookmark *child = node->data;
+		const gchar *uri = kz_bookmark_get_link(child);
+		if (uri && key_uri && !g_strcmp(uri, key_uri))
+		{
+			bookmark = child;	
+			break;
+		}
+
+		if (kz_bookmark_is_folder(child))
+		{
+			KzBookmark *find;
+			find = kz_bookmark_find_bookmark_from_uri(child,
+								  key_uri);
+			if (find)
+			{
+				bookmark = find;
+				break;
+			}
+		}
+	}
+
+	return bookmark;
+}
+
