@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark.h,v 1.19 2004/07/15 13:19:53 ikezoe Exp $
+ *  $Id: kz-bookmark.h,v 1.21 2004/09/22 07:01:00 ikezoe Exp $
  */
 
 #ifndef __KZ_BOOKMARK_H__
@@ -151,6 +151,8 @@ const gchar    *kz_bookmark_get_description	(KzBookmark  *bookmark);
 const gchar    *kz_bookmark_get_location	(KzBookmark  *bookmark);
 const gchar    *kz_bookmark_get_file_type	(KzBookmark  *bookmark);
 guint		kz_bookmark_get_last_modified	(KzBookmark  *bookmark);
+guint		kz_bookmark_get_last_visited	(KzBookmark  *bookmark);
+guint		kz_bookmark_get_added_time	(KzBookmark  *bookmark);
 guint		kz_bookmark_get_interval	(KzBookmark  *bookmark);
 gboolean        kz_bookmark_is_file             (KzBookmark  *bookmark);
 gboolean        kz_bookmark_is_editable         (KzBookmark  *bookmark);
@@ -170,6 +172,10 @@ void		kz_bookmark_set_file_type	(KzBookmark  *bookmark,
 						 const gchar *file_type);
 void		kz_bookmark_set_last_modified	(KzBookmark  *bookmark,
 						 guint        time);
+void		kz_bookmark_set_last_visited	(KzBookmark  *bookmark,
+						 guint        time);
+void		kz_bookmark_set_added_time	(KzBookmark  *bookmark,
+						 guint        time);
 void		kz_bookmark_set_interval	(KzBookmark  *bookmark,
 						 guint        interval);
 
@@ -203,6 +209,8 @@ void		kz_bookmark_sort		(KzBookmark  *bookmark,
 void		kz_bookmark_regist_sort_func	(const gchar *type,
 						 GCompareFunc *func);
 
+KzBookmark *    kz_bookmark_find_bookmark_from_uri (KzBookmark *bookmark,
+						    const gchar *key_uri);
 /* load/save interface */
 /* async */
 gboolean	kz_bookmark_load_start		 (KzBookmark *bookmark);
