@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmarks-view.c,v 1.13 2004/07/23 03:20:55 ikezoe Exp $
+ *  $Id: kz-bookmarks-view.c,v 1.16 2004/08/31 18:04:57 ikezoe Exp $
  */
 
 #include "kz-bookmarks-view.h"
@@ -109,7 +109,7 @@ static GtkTargetEntry dnd_types[] = {
 static const gint dnd_types_num = G_N_ELEMENTS(dnd_types);
 
 
-static GtkWindowClass *parent_class = NULL;
+static GtkTreeViewClass *parent_class = NULL;
 
 
 KZ_OBJECT_GET_TYPE(kz_bookmarks_view, "KzBookmarksView", KzBookmarksView,
@@ -531,10 +531,12 @@ insert_bookmark(KzBookmarksView *view,
 			? KZ_STOCK_REMOTE_BOOKMARK : KZ_STOCK_FOLDER;
 		image = gtk_image_new_from_stock(icon,
 						 GTK_ICON_SIZE_MENU);
+		g_object_ref(G_OBJECT(image));
+		gtk_object_sink(GTK_OBJECT(image));
 		favicon = gtk_widget_render_icon(image, icon,
 						 GTK_ICON_SIZE_MENU,
 						 NULL);
-		gtk_widget_destroy(image);
+		g_object_unref(G_OBJECT(image));
 	}
 	else if (kz_bookmark_is_smart(bookmark))
 	{
@@ -542,10 +544,12 @@ insert_bookmark(KzBookmarksView *view,
 
 		image = gtk_image_new_from_stock(KZ_STOCK_SMART_BOOKMARK,
 						 GTK_ICON_SIZE_MENU);
+		g_object_ref(G_OBJECT(image));
+		gtk_object_sink(GTK_OBJECT(image));
 		favicon = gtk_widget_render_icon(image, KZ_STOCK_SMART_BOOKMARK,
 						 GTK_ICON_SIZE_MENU,
 						 NULL);
-		gtk_widget_destroy(image);
+		g_object_unref(image);
 	}
 	else if (kz_bookmark_is_separator(bookmark))
 	{
@@ -563,10 +567,12 @@ insert_bookmark(KzBookmarksView *view,
 			GtkWidget *image;
 			image = gtk_image_new_from_stock(KZ_STOCK_BOOKMARK,
 							 GTK_ICON_SIZE_MENU);
+			g_object_ref(G_OBJECT(image));
+			gtk_object_sink(GTK_OBJECT(image));
 			favicon = gtk_widget_render_icon(image, KZ_STOCK_BOOKMARK,
 							 GTK_ICON_SIZE_MENU,
 							 NULL);
-			gtk_widget_destroy(image);
+			g_object_unref(image);
 		}
 		g_object_unref(kz_favicon);
 	}
@@ -651,7 +657,6 @@ remove_bookmark(KzBookmarksView *view,
 	
 	treeview = GTK_TREE_VIEW(view);
 	model = gtk_tree_view_get_model(treeview);
-	gtk_widget_grab_focus(GTK_WIDGET(treeview));
 	/* get iter */
 	if (bookmark)
 	{
@@ -859,8 +864,8 @@ connect_bookmark_signals (KzBookmarksView *view,
 
 	g_signal_connect_after(G_OBJECT(bookmark), "insert-child",
 			       G_CALLBACK(cb_bookmark_insert_child), view);
-	g_signal_connect_after(G_OBJECT(bookmark), "remove-child",
-			       G_CALLBACK(cb_bookmark_remove_child), view);
+	g_signal_connect(G_OBJECT(bookmark), "remove-child",
+			 G_CALLBACK(cb_bookmark_remove_child), view);
 	g_signal_connect(G_OBJECT(bookmark), "notify",
 			 G_CALLBACK(cb_bookmark_notify), view);
 
@@ -913,7 +918,8 @@ cb_bookmark_title_edited (GtkCellRendererText *cell,
 	g_return_if_fail(KZ_IS_BOOKMARKS_VIEW(view));
 
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
-        gtk_tree_model_get_iter_from_string(model, &iter, path_str);
+        if(!gtk_tree_model_get_iter_from_string(model, &iter, path_str))
+		return;
 	gtk_tree_model_get(model, &iter,
 			   COLUMN_BOOKMARK, &bookmark,
 			   TERMINATOR);
