@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-root-bookmark.c,v 1.7 2004/07/08 08:01:24 ikezoe Exp $
+ *  $Id: kz-root-bookmark.c,v 1.8 2004/09/18 04:36:57 ikezoe Exp $
  */
 
 #include "kz-root-bookmark.h"
@@ -59,7 +59,7 @@ static void
 kz_root_bookmark_init (KzRootBookmark *root)
 {
 	root->menu          = kz_bookmark_file_new(NULL, NULL, NULL);
-	root->scrap         = kz_bookmark_file_new(NULL, NULL, NULL);
+	root->clip          = kz_bookmark_file_new(NULL, NULL, NULL);
 	root->bookmark_bars = kz_bookmark_pure_folder_new();
 	root->history       = kz_history_new(NULL);
 	root->sessions      = kz_bookmark_pure_folder_new();
@@ -107,10 +107,10 @@ kz_root_bookmark_dispose (GObject *object)
 		g_object_unref(G_OBJECT(root->menu));
 		root->menu = NULL;
 	}
-	if (root->scrap)
+	if (root->clip)
 	{
-		g_object_unref(G_OBJECT(root->scrap));
-		root->scrap = NULL;
+		g_object_unref(G_OBJECT(root->clip));
+		root->clip = NULL;
 	}
 	if (root->bookmark_bars)
 	{
@@ -150,7 +150,7 @@ kz_root_bookmark_dispose (GObject *object)
 
 KzRootBookmark *
 kz_root_bookmark_new (const gchar *menu_file,  const gchar *sys_menu_file,
-		      const gchar *scrap_file, const gchar *sys_scrap_file)
+		      const gchar *clip_file, const gchar *sys_clip_file)
 {
 	KzRootBookmark *root;
 
@@ -161,8 +161,8 @@ kz_root_bookmark_new (const gchar *menu_file,  const gchar *sys_menu_file,
 
 	if (root->menu)
 		kz_bookmark_append(KZ_BOOKMARK(root), root->menu);
-	if (root->scrap)
-		kz_bookmark_append(KZ_BOOKMARK(root), root->scrap);
+	if (root->clip)
+		kz_bookmark_append(KZ_BOOKMARK(root), root->clip);
 	if (root->bookmark_bars)
 		kz_bookmark_append(KZ_BOOKMARK(root), root->bookmark_bars);
 /*
@@ -173,7 +173,7 @@ kz_root_bookmark_new (const gchar *menu_file,  const gchar *sys_menu_file,
 	*/
 
 	kz_root_bookmark_set_menu_file(root, menu_file, sys_menu_file);
-	kz_root_bookmark_set_scrap_file(root, scrap_file, sys_scrap_file);
+	kz_root_bookmark_set_clip_file(root, clip_file, sys_clip_file);
 
 	return root;
 }
@@ -234,36 +234,36 @@ kz_root_bookmark_set_menu_file (KzRootBookmark *root,
 	{
 		root->menu = menu;
 		kz_bookmark_insert_before(KZ_BOOKMARK(root),
-					  menu, root->scrap);
+					  menu, root->clip);
 	}
 }
 
 
 void
-kz_root_bookmark_set_scrap_file (KzRootBookmark *root,
+kz_root_bookmark_set_clip_file (KzRootBookmark *root,
 				 const gchar *file, const gchar *subfile)
 {
-	KzBookmark *scrap;
+	KzBookmark *clip;
 
 	g_return_if_fail(KZ_IS_ROOT_BOOKMARK(root));
 	g_return_if_fail(file && *file);
 
 	/* FIXME! Check permission */
 
-	scrap = ensure_bookmark_file(file, subfile, _("Scrap Bookmarks"), "SCRAPBOOKMARK");
+	clip = ensure_bookmark_file(file, subfile, _("Clip"), "XBEL");
 
-	if (root->scrap)
+	if (root->clip)
 	{
-		kz_bookmark_remove(KZ_BOOKMARK(root), root->scrap);
-		g_object_unref(G_OBJECT(root->scrap));
-		root->scrap = NULL;
+		kz_bookmark_remove(KZ_BOOKMARK(root), root->clip);
+		g_object_unref(G_OBJECT(root->clip));
+		root->clip = NULL;
 	}
 
-	if (scrap)
+	if (clip)
 	{
-		root->scrap = scrap;
+		root->clip = clip;
 		kz_bookmark_insert_before(KZ_BOOKMARK(root),
-					  scrap, root->smarts);
+					  clip, root->smarts);
 	}
 }
 
@@ -347,8 +347,8 @@ kz_root_bookmark_save_all (KzRootBookmark *root)
 
 	if (root->menu)
 		kz_bookmark_save(root->menu);
-	if (root->scrap)
-		kz_bookmark_save(root->scrap);
+	if (root->clip)
+		kz_bookmark_save(root->clip);
 
 	if (root->bookmark_bars)
 	{
