@@ -17,12 +17,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-xbel.c,v 1.42 2004/07/15 13:19:53 ikezoe Exp $
+ *  $Id: kz-xbel.c,v 1.47 2004/09/17 01:28:26 ikezoe Exp $
  */
 
 #include "kazehakase.h"
 #include "kz-xbel.h"
 #include "kz-xml.h"
+#include "utils.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -172,7 +173,7 @@ kz_xbel_from_string (KzBookmark *bookmark,
 		node = kz_xml_element_node_new("xbel");
 		kz_xml_node_set_attr(node, "version", "1.0");
 		kz_xml_node_set_attr(node, "folded", "none");	
-		kz_xml_node_set_attr(node, "xmlns", KAZEHAKASE_URI"2004");	
+		kz_xml_node_set_attr(node, "xmlns:kz", KAZEHAKASE_URI"2004");	
 		/* kz_xml_node_set_attr(node, "id", "????????"); */
 		kz_xml_node_append_child(xml->root, node);
 
@@ -254,6 +255,7 @@ kz_xbel_notify (GObject *object, GParamSpec *pspec, KzXML *xml)
 		xml_node_set_title(node, title);
 		g_free(title);
 	}
+	g_value_unset(&value);
 #if 0
 	else if (!strcmp(prop, "link"))
 	{
@@ -366,6 +368,7 @@ parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
 			}
 
 			{
+#if 1
 				/* 
 				 * for compatibility, 
 				 * we need to load "update_interval" 
@@ -376,6 +379,7 @@ parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
 							   "update_interval");
 				if (interval_node)
 				{
+					/*
 					gchar *str;
 					str = kz_xml_node_to_str(interval_node);		
 					interval = atoi(str);
@@ -385,12 +389,19 @@ parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
 							     "kz:update_interval",
 							     str);
 					g_free(str);
-#if 0
+					*/
 					/* clean "update_interval" node */
+					KzXMLNode *next = kz_xml_node_next(interval_node);
 					kz_xml_node_remove_child(node, interval_node);
 					kz_xml_node_unref(interval_node);
-#endif
+					/* space is also removed */
+					if (kz_xml_node_is_space(next))
+					{
+						kz_xml_node_remove_child(node, next);
+						kz_xml_node_unref(next);
+					}
 				}
+#endif
 			}
 			/*
 			if (kz_bookmark_get_owner(bookmark))
@@ -453,22 +464,35 @@ xml_node_has_location (KzXMLNode *parent)
 	child = xml_node_find_metadata_node(parent);
 	if (!child) return FALSE;
 
+#if 1
+	gchild = xml_node_get_named_node(child, "location");
+	if (gchild)
+	{	
+		KzXMLNode *next = kz_xml_node_next(gchild);
+		/* remove "location" element */
+		kz_xml_node_remove_child(child, gchild);
+		kz_xml_node_unref(gchild);
+		
+		/* space is also removed */
+		if (kz_xml_node_is_space(next))
+		{
+			kz_xml_node_remove_child(child, next);
+			kz_xml_node_unref(next);
+		}
+	}
+#endif
 	if (kz_xml_node_get_attr(child, "kz:location"))
 		return TRUE;
-
-	gchild = xml_node_get_named_node(child, "location");
-	if (gchild) return TRUE;
-
+	
 	return FALSE;
 }
 
 
-static gchar *
+static const gchar *
 xml_node_get_location (KzXMLNode *parent)
 {
-	KzXMLNode *child, *gchild;
+	KzXMLNode *child;
 	const gchar *attr;
-	gchar *location;
 
 	g_return_val_if_fail(parent, FALSE);
 
@@ -480,9 +504,10 @@ xml_node_get_location (KzXMLNode *parent)
 
 	/* Does metadata node have location attr? */
 	attr = kz_xml_node_get_attr(child, "kz:location");
-	if (attr)
-		return g_strdup(attr);
-
+	if (!attr) return NULL;
+	
+	return attr;
+#if 0
 	gchild = xml_node_get_named_node(child, "location");
 	if (!gchild) return NULL;
 
@@ -490,12 +515,13 @@ xml_node_get_location (KzXMLNode *parent)
 	/* set location into metadata attribute */
 	kz_xml_node_set_attr(child, 
 			     "kz:location", location);
-#if 0
+
 	/* remove "location" element */
 	kz_xml_node_remove_child(child, gchild);
 	kz_xml_node_unref(gchild);
-#endif
+
 	return location;
+#endif
 }
 
 
@@ -550,12 +576,11 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			if (xml_node_has_location(node))
 			{
 #warning FIXME! If the file does not exit, we should creat it.
-				gchar *location = xml_node_get_location(node);
+				const gchar *location = xml_node_get_location(node);
 
 				folder = kz_bookmark_file_new(location,
 							      NULL, NULL);
 				kz_bookmark_load_start(folder);
-				g_free(location);
 			}
 			else
 			{
@@ -578,12 +603,37 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 		else if (kz_xml_node_name_is(node, "bookmark"))
 		{
 			KzBookmark *child_bookmark = kz_bookmark_new();
-			const gchar *uri = kz_xml_node_get_attr(node, "href");
+			const GList *attrs = kz_xml_node_get_attrs(node);
+			const GList *list;
 
 			BOOKMARK_SET_BUILDING(bookmark);
 			g_object_set_qdata(G_OBJECT(child_bookmark),
 					   node_quark, node);
-			kz_bookmark_set_link(child_bookmark, uri);
+			for (list = attrs; list; list = g_list_next(list))
+			{
+				KzXMLAttr *attr = list->data;
+				const gchar *name = attr->name;
+
+				if (!strcmp(name, "href"))
+				{
+					kz_bookmark_set_link(child_bookmark, attr->value);
+				}
+				else if (!strcmp(name, "added"))
+				{
+					guint t = 0;
+					if (str_isdigit(attr->value))
+						t = atoi(attr->value);	
+					kz_bookmark_set_added_time(child_bookmark, t);
+				}
+				else if (!strcmp(name, "visited"))
+				{
+					guint t = 0;
+					if (str_isdigit(attr->value))
+						t = atoi(attr->value);	
+					kz_bookmark_set_last_visited(child_bookmark, t);
+				}
+			}
+
 			parse_child_node(child_bookmark, node);
 			/*
 			 * To append child is after parsing child's nodes,
@@ -931,7 +981,8 @@ create_xml_node (KzBookmark *bookmark)
 	else
 	{
 		const gchar *uri = kz_bookmark_get_link(bookmark);
-
+		guint visited = kz_bookmark_get_last_visited(bookmark);
+		guint added = kz_bookmark_get_added_time(bookmark);
 		/*
 		 * <bookmark>\n
 		 * <title/>
@@ -941,6 +992,18 @@ create_xml_node (KzBookmark *bookmark)
 		node = kz_xml_element_node_new("bookmark");
 		if (uri)
 			kz_xml_node_set_attr(node, "href", uri);
+		if (visited != 0)
+		{
+			gchar *str = g_strdup_printf("%d", visited);
+			kz_xml_node_set_attr(node, "visited", str);
+			g_free(str);
+		}
+		if (added != 0)
+		{
+			gchar *str = g_strdup_printf("%d", added);
+			kz_xml_node_set_attr(node, "added", str);
+			g_free(str);
+		}
 
 		space = kz_xml_text_node_new("\n");
 		kz_xml_node_append_child(node, space);
@@ -1234,12 +1297,27 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec)
 		xml_node_set_interval(node, bookmark);
 		/* xml_node_set_metadata(node, "update_interval", str); */
 	}
+	else if (!strcmp(prop, "last-visited"))
+	{
+		guint t = g_value_get_uint(&value);
+		gchar *t_str = g_strdup_printf("%d", t);
+		kz_xml_node_set_attr(node, "visited", t_str);
+		g_free(t_str);
+	}
+	else if (!strcmp(prop, "added-time"))
+	{
+		guint t = g_value_get_uint(&value);
+		gchar *t_str = g_strdup_printf("%d", t);
+		kz_xml_node_set_attr(node, "added", t_str);
+		g_free(t_str);
+	}
 	else if (!strcmp(prop, "smart-list"))
 	{
 		const GList *smart_list;
 		smart_list =  g_value_get_pointer(&value);
 		xml_node_set_smart_list(node, bookmark, smart_list);
 	}
+	g_value_unset(&value);
 }
 
 
