@@ -29,13 +29,16 @@
 #include <glib.h>
 #include "kz-profile.h"
 #include "bookmarks/kz-root-bookmark.h"
-#if 0
+#if USE_MIGEMO
 #include "migemo.h"
 #endif
 G_BEGIN_DECLS
 
 #define KAZEHAKASE_URI "http://kazehakase.sourceforge.jp/"
 
+#define HISTORY_INDEX "/.kazehakase/history_index "
+#define HISTORY_DIR   "/.kazehakase/history/"
+
 
 typedef enum {
 	KZ_UI_LEVEL_INVALID  = 0,
@@ -57,9 +60,13 @@ extern KzProfile      *kz_global_profile;
 extern KzProfile      *kz_session;
 extern KzProfile      *kz_proxy;
 extern KzRootBookmark *kz_bookmarks;
-#if 0
+#if USE_MIGEMO
 extern Migemo         *migemo;
 #endif
+
+extern gboolean        exists_estindex; 
+extern gchar          *estversion;
+
 #define KZ_CONF_GET(s, k, v, t)					\
 	kz_profile_get_value(kz_global_profile,			\
 			     s, k, &(v), sizeof(v),		\
