@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-menu.c,v 1.38 2004/07/05 09:24:06 ikezoe Exp $
+ *  $Id: kz-bookmark-menu.c,v 1.39 2004/08/28 03:49:32 ikezoe Exp $
  */
 
 #include "kz-bookmark-menu.h"
@@ -463,9 +463,7 @@ cb_menu_item_activate (GtkWidget *widget, KzWindow *kz)
 		{
 			GtkWidget *editor;
 
-			editor = kz_bookmark_editor_new(kz_bookmarks->menu);
-			kz_bookmark_editor_folder_view_select
-				(KZ_BOOKMARK_EDITOR(editor), item);
+			editor = kz_bookmark_editor_new(item);
 			gtk_widget_show(editor);
 		}
 		break;
