@@ -23,6 +23,7 @@
 #include "kz-download-box.h"
 #include "kz-actions-download.h"
 #include "gobject-utils.h"
+#include "gtk-utils.h"
 #include "intl.h"
 
 static void kz_download_box_class_init (KzDownloadBoxClass *klass);
@@ -30,7 +31,7 @@ static void kz_download_box_init       (KzDownloadBox      *dlbox);
 static void kz_download_box_dispose    (GObject            *object);
 
 static void kz_download_box_drag_data_received (GtkWidget        *widget,
-						GdkDragContext   *drag_context,
+						GdkDragContext   *context,
 						gint              x,
 						gint              y,
 						GtkSelectionData *data,
@@ -101,7 +102,7 @@ kz_download_box_init (KzDownloadBox *dlbox)
 			  GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_MOTION | 
 			  GTK_DEST_DEFAULT_DROP,
                           url_drag_types,n_url_drag_types,
-			  GDK_ACTION_MOVE);
+			  GDK_ACTION_LINK);
 }
 
 
@@ -110,7 +111,7 @@ kz_download_box_new (KzWindow *kz)
 {
 	KzDownloadBox *dlbox;
 
-	dlbox = KZ_DOWNLOAD_BOX(g_object_new(kz_download_box_get_type(), NULL));
+	dlbox = KZ_DOWNLOAD_BOX(g_object_new(KZ_TYPE_DOWNLOAD_BOX, NULL));
 
 	dlbox->kz = kz;
 	
@@ -132,23 +133,26 @@ kz_download_box_dispose (GObject *object)
 {
 	KzDownloadBox *dlbox = KZ_DOWNLOAD_BOX(object);
 
-	kz_downloader_group_dispose(G_OBJECT(dlbox->dlgrp));
-
 	if (dlbox->icon_list)
 	{
 		g_slist_free(dlbox->icon_list);
 		dlbox->icon_list = NULL;
 	}
-
-	g_signal_handlers_disconnect_by_func(G_OBJECT(dlbox->dlgrp),
-					     G_CALLBACK(cb_add_item),
-					     dlbox);
-	g_signal_handlers_disconnect_by_func(G_OBJECT(dlbox->dlgrp),
-					     G_CALLBACK(cb_remove_item),
-					     dlbox);
-	g_signal_handlers_disconnect_by_func(G_OBJECT(dlbox->dlgrp),
-					     G_CALLBACK(cb_progress_item),
-					     dlbox);
+	
+	if (dlbox->dlgrp)
+	{
+		g_signal_handlers_disconnect_by_func(G_OBJECT(dlbox->dlgrp),
+						     G_CALLBACK(cb_add_item),
+						     dlbox);
+		g_signal_handlers_disconnect_by_func(G_OBJECT(dlbox->dlgrp),
+						     G_CALLBACK(cb_remove_item),
+						     dlbox);
+		g_signal_handlers_disconnect_by_func(G_OBJECT(dlbox->dlgrp),
+						     G_CALLBACK(cb_progress_item),
+						     dlbox);
+		g_object_unref(dlbox->dlgrp);
+		dlbox->dlgrp = NULL;
+	}
 
 	if (G_OBJECT_CLASS (parent_class)->dispose)
 		(*G_OBJECT_CLASS (parent_class)->dispose) (object);
@@ -157,17 +161,16 @@ kz_download_box_dispose (GObject *object)
 
 static void
 kz_download_box_drag_data_received (GtkWidget *widget,
-				    GdkDragContext *drag_context,
+				    GdkDragContext *context,
 				    gint x, gint y,
 				    GtkSelectionData *data,
 				    guint info,
 				    guint time)
 {
 	KzDownloadBox *dlbox;
-	KzDownloader *dl;
 	gchar **strings, *basename;
 
-	g_return_if_fail(KZ_DOWNLOAD_BOX(widget));
+	g_return_if_fail(KZ_IS_DOWNLOAD_BOX(widget));
 	dlbox = KZ_DOWNLOAD_BOX(widget);
 
 	switch (info)
@@ -182,8 +185,8 @@ kz_download_box_drag_data_received (GtkWidget *widget,
 		if (g_file_test(basename, G_FILE_TEST_EXISTS))
 		{
 			gboolean ret;
-			ret = kz_actions_download_confirm_replace(GTK_WINDOW(dlbox->kz),
-								  basename);
+			ret = gtkutil_confirm_file_replace(GTK_WINDOW(dlbox->kz),
+							   basename);
 			if (!ret)
 			{
 				kz_actions_download_open_save_dialog(strings[0], FALSE);
@@ -193,19 +196,17 @@ kz_download_box_drag_data_received (GtkWidget *widget,
 			}
 		}
 		
-		dl = kz_downloader_new_with_filename(strings[0], basename);
-		if (dl)
-		{
-			kz_downloader_group_add_item(dlbox->dlgrp, dl);
-			kz_downloader_to_file(dl);
-		}
+		kz_downloader_group_add_item(dlbox->dlgrp, strings[0]);
 		
 		g_strfreev(strings);
 		g_free(basename);
+		gtk_drag_finish(context, TRUE, FALSE, time);
+		return;
 		break;
 	 default:
 		break;
 	}
+	gtk_drag_finish(context, FALSE, FALSE, time);
 }
 
 
@@ -229,9 +230,7 @@ cb_add_item (KzDownloaderGroup *dlgrp,
 	GtkIconSet *icon_set;
 
 	icon = g_new0(KzDownloaderIcon, 1);
-	icon->dlbox = dlbox;
-	icon->dl    = dl;
-	g_object_ref(icon->dl);
+	icon->dl    = g_object_ref(dl);
 
 	icon->fraction = 0;
 
@@ -239,6 +238,7 @@ cb_add_item (KzDownloaderGroup *dlgrp,
 	icon->event_box   = gtk_event_box_new();
 	icon->image = gtk_image_new();
 	icon->tips  = gtk_tooltips_new();
+	dlbox->icon_list = g_slist_append(dlbox->icon_list, icon);
 
 	/* create semitransparent icon image */
 	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
@@ -280,8 +280,7 @@ cb_add_item (KzDownloaderGroup *dlgrp,
 		
 	gtk_image_set_from_pixbuf(GTK_IMAGE(icon->image), transparent_pixbuf);
 	g_object_unref(orig_pixbuf);
-
-	dlbox->icon_list = g_slist_append(dlbox->icon_list, icon);
+	g_object_unref(transparent_pixbuf);
 
 	gtk_container_add(GTK_CONTAINER(icon->event_box), icon->image);
 	gtk_widget_show(icon->image);
@@ -301,16 +300,16 @@ cb_remove_item (KzDownloaderGroup *dlgrp,
 {
 	GSList *list;
 
-	g_return_if_fail(KZ_DOWNLOADER(dl));
-	g_return_if_fail(KZ_DOWNLOADER_GROUP(dlgrp));
-	g_return_if_fail(KZ_DOWNLOAD_BOX(dlbox));
+	g_return_if_fail(KZ_IS_DOWNLOADER(dl));
+	g_return_if_fail(KZ_IS_DOWNLOADER_GROUP(dlgrp));
+	g_return_if_fail(KZ_IS_DOWNLOAD_BOX(dlbox));
 
 	for (list = dlbox->icon_list; list; list = g_slist_next(list))
 	{
 		KzDownloaderIcon *icon = list->data;
 		if (icon->dl == dl)
 		{
-			g_slist_remove(dlbox->icon_list, icon);
+			dlbox->icon_list = g_slist_remove(dlbox->icon_list, icon);
 			remove_icon(icon);
 			g_object_unref(icon->dl);
 			icon->dl = NULL;
@@ -333,9 +332,9 @@ cb_progress_item (KzDownloaderGroup *dlgrp,
 	KzDownloaderIcon *icon = NULL;
 	gint width, height, fraction;
 
-	g_return_if_fail(KZ_DOWNLOADER(dl));
-	g_return_if_fail(KZ_DOWNLOADER_GROUP(dlgrp));
-	g_return_if_fail(KZ_DOWNLOAD_BOX(dlbox));
+	g_return_if_fail(KZ_IS_DOWNLOADER(dl));
+	g_return_if_fail(KZ_IS_DOWNLOADER_GROUP(dlgrp));
+	g_return_if_fail(KZ_IS_DOWNLOAD_BOX(dlbox));
 
 	/* find icon */
 	for (list = dlbox->icon_list; list; list = g_slist_next(list))
@@ -390,7 +389,7 @@ cb_progress_item (KzDownloaderGroup *dlgrp,
 			else
 			{
 				g_warning(_("KzDownloadBox: "
-					    "unknown icon fade in direction!: %s"),
+					    "unknown the direction of icon fading in!: %s"),
 					  direction);
 				y = height - height * percent / 100;
 				height = height * percent / 100;
