@@ -23,6 +23,7 @@
 #include "kz-profile.h"
 #include "kz-downloader.h"
 #include "utils.h"
+#include "glib-utils.h"
 
 struct _KzFaviconPriv
 {
@@ -38,7 +39,6 @@ enum {
 };
 
 #define MAX_FAVICONS 2000
-#define g_strcmp(s1,s2) ((s1)!=NULL && (s2)!=NULL?strcmp(s1,s2):0)
 
 static void kz_favicon_class_init   (KzFaviconClass *klass);
 static void kz_favicon_init         (KzFavicon *kzfav);
@@ -101,9 +101,10 @@ kz_favicon_init (KzFavicon *kzfav)
 	/* make dir which stores icon file */
 	make_dir("favicon");
 
-	favicon_file = g_strconcat(g_get_home_dir(),
-				   "/.kazehakase/faviconrc",
-				   NULL);
+	favicon_file = g_build_filename(g_get_home_dir(),
+				        ".kazehakase",
+					"faviconrc",
+				   	NULL);
 	kzfav->priv->uri_list = kz_profile_open(favicon_file, NULL);
 	g_free(favicon_file);
 
@@ -125,9 +126,10 @@ kz_favicon_dispose (GObject *object)
 	g_return_if_fail(KZ_IS_FAVICON(object));
 
 	kzfav = KZ_FAVICON(object);
-	favicon_file = g_strconcat(g_get_home_dir(),
-				   "/.kazehakase/faviconrc",
-				   NULL);
+	favicon_file = g_build_filename(g_get_home_dir(),
+				   	".kazehakase",
+					"faviconrc",
+				   	NULL);
 	
 	kzfav->priv->uri_list = kz_profile_open(favicon_file, NULL);
 	g_free(favicon_file);
@@ -230,10 +232,11 @@ cb_favicon_uri_notify (KzMozEmbed *kzembed,
 	hash_key = create_profile_key_from_uri(location);
 
 	favicon_filename = create_filename_from_uri(favicon_uri);
-	favicon_file_location = g_strconcat(g_get_home_dir(),
-					    "/.kazehakase/favicon/",
-					    favicon_filename,
-					    NULL);
+	favicon_file_location = g_build_filename(g_get_home_dir(),
+					    	 ".kazehakase",
+						 "favicon",
+					    	 favicon_filename,
+					    	 NULL);
 	g_free(favicon_filename);
 	
 	old_value = g_hash_table_lookup(kzfav->priv->uri_hash,
