@@ -41,7 +41,8 @@
 #include <stdio.h>
 #include <string.h>
 #include "gobject-utils.h"
-#include "egg-marshalers.h"
+#include "glib-utils.h"
+#include "kz-marshalers.h"
 
 
 enum {
@@ -54,10 +55,6 @@ enum {
 };
 
 
-#define g_strlen(s) ((s)!=NULL?strlen(s):0)
-#define g_strcmp(s1,s2) ((s1)!=NULL && (s2)!=NULL?strcmp(s1,s2):0)
-
-
 static void kz_profile_class_init (KzProfileClass *klass);
 static void kz_profile_init       (KzProfile      *profile);
 static void kz_profile_dispose    (GObject        *object);
@@ -115,7 +112,7 @@ kz_profile_class_init (KzProfileClass *klass)
 				G_SIGNAL_RUN_FIRST | G_SIGNAL_DETAILED,
 				G_STRUCT_OFFSET (KzProfileClass, key_added),
 				NULL, NULL,
-				_egg_marshal_VOID__STRING_STRING,
+				_kz_marshal_VOID__STRING_STRING,
 				G_TYPE_NONE, 2,
 				G_TYPE_STRING, G_TYPE_STRING);
 
@@ -125,7 +122,7 @@ kz_profile_class_init (KzProfileClass *klass)
 				G_SIGNAL_RUN_FIRST | G_SIGNAL_DETAILED,
 				G_STRUCT_OFFSET (KzProfileClass, key_deleted),
 				NULL, NULL,
-				_egg_marshal_VOID__STRING_STRING,
+				_kz_marshal_VOID__STRING_STRING,
 				G_TYPE_NONE, 2,
 				G_TYPE_STRING, G_TYPE_STRING);
 
@@ -135,7 +132,7 @@ kz_profile_class_init (KzProfileClass *klass)
 				G_SIGNAL_RUN_FIRST | G_SIGNAL_DETAILED,
 				G_STRUCT_OFFSET (KzProfileClass, changed),
 				NULL, NULL,
-				_egg_marshal_VOID__STRING_STRING_STRING,
+				_kz_marshal_VOID__STRING_STRING_STRING,
 				G_TYPE_NONE, 3,
 				G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 }
@@ -524,7 +521,7 @@ kz_profile_get_string (KzProfile *profile,
 	g_return_val_if_fail(KZ_IS_PROFILE(profile), NULL);
 
 	if (!profile || !section || !key)
-		return FALSE;
+		return NULL;
 	for (p = profile->list; p; p = p->next)
 	{
 		if (p->type == KZ_PROFILE_DATA_TYPE_KEY 
@@ -1011,7 +1008,9 @@ compare_enum_key (gconstpointer a, gconstpointer b)
  * RET,list of keys,NULL:error
 */
 GList *
-kz_profile_enum_key (KzProfile *profile, const gchar *section)
+kz_profile_enum_key (KzProfile *profile, 
+		     const gchar *section,
+		     gboolean users_only)
 {
 	GList *glist = NULL;
 	KzProfileList *p;
@@ -1029,6 +1028,12 @@ kz_profile_enum_key (KzProfile *profile, const gchar *section)
 						     (GCompareFunc)compare_enum_key);
 		}
 	}
+
+	/* We need only user's setting if it exists */
+	
+	if (glist && users_only)
+		return glist;
+	
 	for (p = profile->sublist; p; p = p->next)
 	{
 		if (p->section && p->key
