@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-sidebar.c,v 1.12 2004/04/18 12:17:23 makeinu Exp $
+ *  $Id: kz-sidebar.c,v 1.13 2004/08/10 04:47:15 ikezoe Exp $
  */
 
 #include "kz-sidebar.h"
@@ -30,6 +30,7 @@
 #include "kz-bookmarks-sidebar.h"
 #include "kz-downloader-sidebar.h"
 #include "kz-tabtree.h"
+#include "kz-thumbnails-sidebar.h"
 
 
 #define ADD_ENTRIES(func) 						\
@@ -108,6 +109,7 @@ kz_sidebar_class_init (KzSidebarClass *klass)
 	ADD_ENTRIES(kz_bookmarks_sidebar_get_entry);
 	ADD_ENTRIES(kz_tab_tree_get_entry);
 	ADD_ENTRIES(kz_downloader_sidebar_get_entry);
+	ADD_ENTRIES(kz_thumbnails_sidebar_get_entry);
 }
 
 static void
