@@ -49,14 +49,7 @@ struct _KzTabLabelPriv
 	gulong favicon_signal;
 };
 
-#warning FIXME!
-
-const gchar *label_color[] =
-{
-	"#000000",
-	"#ff0000",
-	"#22aa44",
-};
+static gchar *label_color[KZ_TAB_LABEL_N_STATE];
 
 /* object class */
 static void     kz_tab_label_class_init         (KzTabLabelClass *klass);
@@ -108,6 +101,8 @@ static void     kz_tab_label_sync_to_profile    (KzTabLabel *kztab);
 static void     kz_tab_label_move_page          (KzTabLabel *kztab,
 						 KzTabLabel *dest_kztab);
 
+static void	kz_tab_label_set_visited        (KzTabLabel *kztab);
+
 static void     cb_close_button_clicked         (GtkWidget *button,
 						 KzTabLabel *kztab);
 
@@ -320,8 +315,8 @@ kz_tab_label_init (KzTabLabel *kztab)
 
 	g_signal_connect(G_OBJECT(kz_global_profile), "changed::Tab",
 			 G_CALLBACK(cb_profile_changed), kztab);
-	kz_tab_label_set_text(kztab, NULL);
 	kz_tab_label_sync_to_profile(kztab);
+	kz_tab_label_set_text(kztab, NULL);
 }
 
 
@@ -377,6 +372,7 @@ kz_tab_label_dispose (GObject *object)
 					     kztab);
 	if (kztab->kzembed)
 	{
+	
 		g_signal_handlers_disconnect_by_func(G_OBJECT(kztab->kzembed),
 						     cb_net_start, kztab);
 		g_signal_handlers_disconnect_by_func(G_OBJECT(kztab->kzembed),
@@ -537,6 +533,7 @@ kz_tab_label_set_state (KzTabLabel *kztab,
 		kztab->state = state;
 		break;
 	case KZ_TAB_LABEL_STATE_NORMAL:
+		kz_tab_label_set_visited(kztab);
 	default:
 		kztab->state = KZ_TAB_LABEL_STATE_NORMAL;
 		break;
@@ -889,6 +886,7 @@ kz_tab_label_sync_to_profile (KzTabLabel *kztab)
 	gboolean fix = TRUE;
 	gboolean show_close = TRUE;
 	gboolean show_favicon = TRUE;
+	gchar *color;
 
 	/* width */
 	KZ_CONF_GET("Tab", "fix_width", fix, BOOL);
@@ -902,13 +900,39 @@ kz_tab_label_sync_to_profile (KzTabLabel *kztab)
 	/* close button */
 	KZ_CONF_GET("Tab", "show_close_button", show_close, BOOL);
 	kz_tab_label_set_show_close_button(kztab, show_close && !kztab->priv->lock);
-	
+
 	/* lock button */
 	kz_tab_label_set_show_lock_button(kztab, kztab->priv->lock);
 
 	/* favicon */
 	KZ_CONF_GET("Tab", "show_favicon", show_favicon, BOOL);
 	kz_tab_label_set_show_favicon(kztab, show_favicon);
+
+	/* label color */
+	color = KZ_CONF_GET_STR("Tab" , "normal_color");
+	if (!color)
+		color = g_strdup("#000000");
+
+	if (label_color[KZ_TAB_LABEL_STATE_NORMAL])
+		g_free(label_color[KZ_TAB_LABEL_STATE_NORMAL]);
+	label_color[KZ_TAB_LABEL_STATE_NORMAL] = g_strdup(color);
+	g_free(color);
+
+	color = KZ_CONF_GET_STR("Tab" , "loading_color");
+	if (!color)
+		color = g_strdup("#ff0000");
+	if (label_color[KZ_TAB_LABEL_STATE_LOADING])
+		g_free(label_color[KZ_TAB_LABEL_STATE_LOADING]);
+	label_color[KZ_TAB_LABEL_STATE_LOADING] = g_strdup(color);
+	g_free(color);
+
+	color = KZ_CONF_GET_STR("Tab" , "loaded_color");
+	if (!color)
+		color = g_strdup("#22aa44");
+	if (label_color[KZ_TAB_LABEL_STATE_LOADED])
+		g_free(label_color[KZ_TAB_LABEL_STATE_LOADED]);
+	label_color[KZ_TAB_LABEL_STATE_LOADED] = g_strdup(color);
+	g_free(color);
 }
 
 /* move kztab to dest_kztab's position */
@@ -916,6 +940,7 @@ static void
 kz_tab_label_move_page(KzTabLabel *kztab, KzTabLabel *dest_kztab)
 {
 	GtkWidget *src_page;
+
 	GtkNotebook *notebook;
 	gint dest_pos;
 
@@ -1227,3 +1252,23 @@ kz_tab_label_set_lock (KzTabLabel *kztab, gboolean lock)
 	kz_moz_embed_set_lock(kztab->kzembed, lock);
 	kz_tab_label_sync_to_profile(kztab);
 }
+
+static void
+kz_tab_label_set_visited (KzTabLabel *kztab)
+{
+	KzBookmark *bookmark = NULL;
+	const gchar *uri;
+	
+	uri = kz_moz_embed_get_location(kztab->kzembed);
+	/* find the bookmark which has the same uri */
+	bookmark = kz_bookmark_find_bookmark_from_uri(kz_bookmarks->menu,
+						      uri);
+	if (bookmark)
+	{
+		GTimeVal now;
+
+		g_get_current_time(&now);
+		kz_bookmark_set_last_visited(bookmark, now.tv_sec);
+	}
+}
+
