@@ -29,10 +29,10 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtkmozembed.h>
 
-#include "egg-toggle-action.h"
-#include "egg-history-action.h"
-#include "egg-toolbar.h"
-#include "egg-marshalers.h"
+#include "gtk24backports.h"
+#include "kz-marshalers.h"
+#include "kz-history-action.h"
+#include "kz-zoom-action.h"
 #include "gobject-utils.h"
 #include "gtk-utils.h"
 #include "kz-actions.h"
@@ -42,7 +42,6 @@
 #include "kz-mozembed.h"
 #include "kz-tab-label.h"
 #include "kz-icons.h"
-#include "google.h"
 #include "kazehakase.h"
 #include "intl.h"
 #include "utils.h"
@@ -54,6 +53,8 @@
 #include "kz-entry.h"
 #include "kz-proxy-menu.h"
 
+#define MAX_CLOSED_TABS 10
+
 enum {
 	APPEND_TAB_SIGNAL,
 	REMOVE_TAB_SIGNAL,
@@ -173,7 +174,16 @@ static void     cb_bookmark_bars_remove_child (KzBookmark      *bookmark,
 					       KzWindow        *kz);
 static void     cb_bookmark_bars_reordered    (KzBookmark      *bookmark,
 					       KzWindow        *kz);
-static void     cb_menu_merge_add_widget      (EggMenuMerge    *merge,
+static void     cb_smartbookmark_insert_child (KzBookmark      *bookmark,
+					       KzBookmark      *child,
+					       KzBookmark      *sibling,
+					       KzWindow        *kz);
+static void     cb_smartbookmark_remove_child (KzBookmark      *bookmark,
+					       KzBookmark      *child,
+					       KzWindow        *kz);
+static void     cb_smartbookmark_reordered    (KzBookmark      *bookmark,
+					       KzWindow        *kz);
+static void     cb_menu_merge_add_widget      (GtkUIManager    *merge,
 					       GtkWidget       *widget,
 					       GtkBox          *box);
 static void     cb_notebook_switch_page       (GtkNotebook     *notebook,
@@ -194,9 +204,9 @@ static gboolean cb_notebook_scroll_event      (GtkWidget       *widget,
 					       GdkEventScroll  *event,
 					       KzWindow        *kz);
 static void     cb_sidebar_map                (GtkWidget       *widget,
-					       EggToggleAction *action);
+					       GtkToggleAction *action);
 static void     cb_sidebar_unmap              (GtkWidget       *widget,
-					       EggToggleAction *action);
+					       GtkToggleAction *action);
 static gboolean cb_focus_out_event            (GtkWidget       *widget,
 					       GdkEventFocus   *event,
 					       KzWindow        *kz);
@@ -298,7 +308,7 @@ kz_window_class_init (KzWindowClass *klass)
 				G_SIGNAL_RUN_FIRST,
 				G_STRUCT_OFFSET (KzWindowClass, append_tab),
 				NULL, NULL,
-				_egg_marshal_VOID__OBJECT_OBJECT,
+				_kz_marshal_VOID__OBJECT_OBJECT,
 				G_TYPE_NONE, 2,
 				GTK_TYPE_WIDGET, GTK_TYPE_WIDGET);
 
@@ -318,7 +328,7 @@ kz_window_class_init (KzWindowClass *klass)
 				G_SIGNAL_RUN_FIRST,
 				G_STRUCT_OFFSET (KzWindowClass, reorder_tab),
 				NULL, NULL,
-				_egg_marshal_VOID__OBJECT_INT,
+				_kz_marshal_VOID__OBJECT_INT,
 				G_TYPE_NONE, 2,
 				GTK_TYPE_WIDGET, G_TYPE_INT);
 
@@ -340,22 +350,20 @@ kz_window_class_init (KzWindowClass *klass)
 static void
 kz_window_init (KzWindow *kz)
 {
-	GtkAccelGroup *accels;
 	GtkWidget *menu_box;
 	GtkSettings *setting;
 	GtkWidget *vseparator;
-	GtkWidget *curpage;
+/*	GtkWidget *curpage; */ 
 	GList *children, *node;
+	GtkAccelGroup *accel_group;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 	
 	g_object_set(G_OBJECT(kz), "allow-shrink", TRUE, NULL);
 
+	gtk_window_set_title(GTK_WINDOW(kz), _("Kazehakase"));
 	gtk_window_set_icon(GTK_WINDOW(kz), kz_icon);
 
-	/* accels */
-	accels = gtk_accel_group_new();
-
 	/* init member */
 	kz->top_vbox           = gtk_vbox_new(FALSE, 0);
 
@@ -375,10 +383,12 @@ kz_window_init (KzWindow *kz)
 	kz->find_direction     = gtk_toggle_button_new();
 	kz->dlbox              = kz_download_box_new(kz);
 
-	kz->actions            = kz_actions_create_group(kz, accels);
-	kz->popup_actions      = kz_actions_popup_create_group(kz, accels);
-	kz->tabpop_actions     = kz_actions_tab_popup_create_group(kz, accels);
-	kz->menu_merge         = egg_menu_merge_new();
+	kz->menu_merge         = gtk_ui_manager_new();
+	accel_group            = gtk_ui_manager_get_accel_group(kz->menu_merge);
+
+	kz->actions            = kz_actions_create_group(kz, accel_group);
+	kz->popup_actions      = kz_actions_popup_create_group(kz, accel_group);
+	kz->tabpop_actions     = kz_actions_tab_popup_create_group(kz, accel_group);
 	
 	kz->kzfav              = kz_favicon_get_instance();
 
@@ -402,6 +412,11 @@ kz_window_init (KzWindow *kz)
 	kz->priv->view_hist    = NULL;
 	kz->priv->tab_tree     = NULL;
 
+	kz->closed_tabs     = kz_bookmark_pure_folder_new();
+	kz->history_search  = kz_bookmark_pure_folder_new();
+
+	kz->thumb_creator   = NULL;
+
 	/* gesture */
 	{
 		kz_window_set_gesture_items(kz);
@@ -430,18 +445,21 @@ kz_window_init (KzWindow *kz)
 				   FALSE, FALSE, 0);   
 		gtk_widget_show(menu_box);
 
-		egg_menu_merge_set_accel_group(kz->menu_merge, accels);
-		egg_menu_merge_insert_action_group(kz->menu_merge,
+		gtk_ui_manager_insert_action_group(kz->menu_merge,
 						   kz->actions, 0);
-		egg_menu_merge_insert_action_group(kz->menu_merge,
+		gtk_ui_manager_insert_action_group(kz->menu_merge,
 						   kz->popup_actions, 0);
-		egg_menu_merge_insert_action_group(kz->menu_merge,
+		gtk_ui_manager_insert_action_group(kz->menu_merge,
 						   kz->tabpop_actions, 0);
+		
 		g_signal_connect(kz->menu_merge, "add_widget",
 				 G_CALLBACK(cb_menu_merge_add_widget),
 				 menu_box);
-
+		gtk_window_add_accel_group(GTK_WINDOW(kz), 
+					   gtk_ui_manager_get_accel_group(kz->menu_merge));
+	
 		kz_window_sync_ui_level(kz);
+		gtk_ui_manager_ensure_update(kz->menu_merge);
 		g_signal_connect(G_OBJECT(kz_global_profile),
 				 "changed::Global",
 				 G_CALLBACK(cb_profile_global_changed), kz);
@@ -452,6 +470,7 @@ kz_window_init (KzWindow *kz)
 
 		/* FIXME */
 		setting = gtk_settings_get_default();
+
 		if (setting)
 		{
 			gtk_settings_set_long_property (setting,
@@ -460,9 +479,20 @@ kz_window_init (KzWindow *kz)
 							"");
 		}
 
-		gtk_window_add_accel_group(GTK_WINDOW(kz), accels);
 	}
 
+	/* smart bookmark */
+	g_signal_connect_after(G_OBJECT(kz_bookmarks->smarts),
+			       "insert-child",
+			       G_CALLBACK(cb_smartbookmark_insert_child), kz);
+	g_signal_connect_after(G_OBJECT(kz_bookmarks->smarts),
+			       "remove-child",
+			       G_CALLBACK(cb_smartbookmark_remove_child), kz);
+	g_signal_connect_after(G_OBJECT(kz_bookmarks->smarts),
+			       "children-reordered",
+			       G_CALLBACK(cb_smartbookmark_reordered), kz);
+	kz_actions_update_smartbookmarks(kz, kz_bookmarks->smarts);
+
 	/* bookmark bar */
 	gtk_box_pack_start(GTK_BOX(kz->top_vbox), 
 			   kz->bookmark_bars_area, FALSE, FALSE, 0);
@@ -519,8 +549,8 @@ kz_window_init (KzWindow *kz)
 		gtk_paned_add1(GTK_PANED(kz->pane), kz->sidebar);
 		/* gtk_widget_show(kz->sidebar); */
 		{
-			EggAction *action;
-			action = egg_action_group_get_action(kz->actions,
+			GtkAction *action;
+			action = gtk_action_group_get_action(kz->actions,
 							     "ShowHideSidebar");
 			g_signal_connect(G_OBJECT(kz->sidebar), "map",
 					 G_CALLBACK(cb_sidebar_map), action);
@@ -529,7 +559,7 @@ kz_window_init (KzWindow *kz)
 		}
 
 		/* mozembed tab */
-		curpage = kz_window_open_new_tab(kz, NULL);
+		/* curpage = kz_window_open_new_tab(kz, NULL); */
 	}
 
 	/* search area */
@@ -553,7 +583,7 @@ kz_window_init (KzWindow *kz)
 		gtk_widget_show(toggle);
 
 		kz_entry_set_backtext(KZ_ENTRY(kz->find_area),
-				      _("Search in this page"));
+				      _("Find in this page"));
 		kz_entry_set_icon_from_stock(KZ_ENTRY(kz->find_area),
 					     GTK_STOCK_FIND,
 					     GTK_ICON_SIZE_MENU);
@@ -583,7 +613,7 @@ kz_window_init (KzWindow *kz)
 		g_object_ref(G_OBJECT(tips));
 		gtk_object_sink(GTK_OBJECT(tips));
 		gtk_tooltips_set_tip(tips, kz->find_area,
-				     _("Find in this page"), NULL);
+				     _("Find a word or phrase in this page"), NULL);
 		kz->priv->find_tips[1] = tips;
 	}
 
@@ -613,13 +643,22 @@ kz_window_init (KzWindow *kz)
 		gtk_widget_show(vseparator);
 	}
 
+	/* a widget for creating thumbnail */
+	{
+		gboolean create_thumbnail = FALSE;
+		KZ_CONF_GET("Global", "create_thumbnail", create_thumbnail, BOOL);
+
+		if (create_thumbnail)
+		{
+			kz_window_create_thumbnail_creator(kz);
+		}
+	}
+
 	window_list = g_list_append(window_list, kz);
 
-	kz_actions_set_sensitive(kz, KZ_MOZ_EMBED(curpage));
+	/* kz_actions_set_sensitive(kz, KZ_MOZ_EMBED(curpage)); */
 	kz_window_restore_state(kz);
 
-	gtk_widget_show(GTK_WIDGET(kz));
-
 	kz_window_restore_session(kz);
 }
 
@@ -628,15 +667,38 @@ static void
 kz_window_append_tab (KzWindow *kz, GtkWidget *widget, GtkWidget *parent)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	kz_actions_set_sensitive(kz, KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	kz_actions_set_tab_sensitive(kz, KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
 }
 
 
 static void
 kz_window_remove_tab (KzWindow *kz, GtkWidget *widget)
 {
+
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 	kz_actions_set_sensitive(kz, KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	kz_actions_set_tab_sensitive(kz, KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+
+	if (KZ_IS_MOZ_EMBED(widget))
+	{
+		KzBookmark *bookmark;
+		const gchar *uri, *title;
+		GList *children;
+
+		uri = kz_moz_embed_get_location(KZ_MOZ_EMBED(widget));
+		title = kz_moz_embed_get_title(KZ_MOZ_EMBED(widget));
+		bookmark = kz_bookmark_new_with_attrs(title, uri, title);
+		kz_bookmark_prepend(kz->closed_tabs, bookmark);
+		g_object_unref(bookmark);
+
+		/* removed the oldest one */
+		children = kz_bookmark_get_children(kz->closed_tabs);
+		bookmark = g_list_nth_data(children, MAX_CLOSED_TABS);
+		if (bookmark)
+			kz_bookmark_remove(kz->closed_tabs, bookmark);
+		if (children)
+			g_list_free(children);
+	}
 }
 
 
@@ -653,12 +715,9 @@ kz_window_new(const gchar *url)
 	KzWindow *kz = KZ_WINDOW(g_object_new(KZ_TYPE_WINDOW,
 					      "type", GTK_WINDOW_TOPLEVEL,
 					      NULL));
-	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
-
-	if (GTK_IS_MOZ_EMBED(widget))
-		/* FIXME! */
-		/* kz_moz_embed_load_url(KZ_MOZ_EMBED(widget), url); */
-		gtk_moz_embed_load_url(GTK_MOZ_EMBED(widget), url);
+	
+	if (url)
+		kz_window_open_new_tab (kz, url);
 
 	return GTK_WIDGET (kz);
 }
@@ -667,11 +726,12 @@ kz_window_new(const gchar *url)
 void
 kz_window_sync_ui_level (KzWindow *kz)
 {
-	EggAction *action;
+	GtkAction *action;
 	const gchar *action_str;
 	GError *err = NULL;
 	gchar *ui_file, *level = NULL;
 	GtkWidget *widget;
+	KzMozEmbed *kzembed;
 	gchar path[64] = {0};
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
@@ -692,16 +752,16 @@ kz_window_sync_ui_level (KzWindow *kz)
 		break;
 	}
 
-	action = egg_action_group_get_action(kz->actions, action_str);
-	if (!egg_toggle_action_get_active(EGG_TOGGLE_ACTION(action)))
-		egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action), TRUE);
+	action = gtk_action_group_get_action(kz->actions, action_str);
+	if (!gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)))
+		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), TRUE);
 
 
 	level = KZ_CONF_GET_STR("Global", "ui_level");
 
 	if (kz->priv->merge_id)
 	{
-		egg_menu_merge_remove_ui(kz->menu_merge, kz->priv->merge_id);
+		gtk_ui_manager_remove_ui(kz->menu_merge, kz->priv->merge_id);
 		kz->priv->merge_id = 0;
 	}
 
@@ -717,19 +777,19 @@ kz_window_sync_ui_level (KzWindow *kz)
 		ui_file = g_strdup(KZ_SYSCONFDIR"/kz-ui-beginner.xml");
 
 	kz->priv->merge_id
-		= egg_menu_merge_add_ui_from_file(kz->menu_merge,
+		= gtk_ui_manager_add_ui_from_file(kz->menu_merge,
 						  ui_file, &err);
 	if (err)
 	{
 		g_warning("%s", err->message);
 		g_error_free(err);
 	}
-        egg_menu_merge_ensure_update(kz->menu_merge);
+        gtk_ui_manager_ensure_update(kz->menu_merge);
 
 	/* update bookmarks */
-	g_snprintf(path, sizeof(path), "/menu/BookmarksMenu%s",
+	g_snprintf(path, sizeof(path), "/menubar/BookmarksMenu%s",
 		   kz_ui_level_to_suffix(kz_ui_level()));
-	widget = egg_menu_merge_get_widget(kz->menu_merge, path);
+	widget = gtk_ui_manager_get_widget(kz->menu_merge, path);
 	if (GTK_IS_MENU_ITEM(widget))
 		widget = gtk_menu_item_get_submenu(GTK_MENU_ITEM(widget));
 	if (GTK_IS_MENU_SHELL(widget))
@@ -739,9 +799,51 @@ kz_window_sync_ui_level (KzWindow *kz)
 						  kz_bookmarks->menu);
 	}
 
-        /* update proxy */
+	/* update clips */
+	g_snprintf(path, sizeof(path), "/menubar/ToolsMenu%s/ClipMenu",
+		   kz_ui_level_to_suffix(kz_ui_level()));
+	widget = gtk_ui_manager_get_widget(kz->menu_merge, path);
+	if (GTK_IS_MENU_ITEM(widget))
+		widget = gtk_menu_item_get_submenu(GTK_MENU_ITEM(widget));
+	if (GTK_IS_MENU_SHELL(widget))
+	{
+		kz_bookmark_menu_remove_menuitems(GTK_MENU_SHELL(widget), kz);
+		kz_bookmark_menu_append_menuitems(GTK_MENU_SHELL(widget), kz,
+						  kz_bookmarks->clip);
+	}
+
+	/* append recent close tab menu */
+	g_snprintf(path, sizeof(path), "/menubar/TabMenu%s/RecentCloseTabMenu",
+		   kz_ui_level_to_suffix(kz_ui_level()));
+	widget = gtk_ui_manager_get_widget(kz->menu_merge, path);
+
+	if (GTK_IS_MENU_ITEM(widget))
+	{
+		GtkWidget *submenu;
+		submenu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(widget));
+		if (!submenu)
+		{
+			submenu = gtk_menu_new();
+			gtk_menu_item_set_submenu(GTK_MENU_ITEM(widget), submenu);
+		}
+		if (GTK_IS_MENU_SHELL(submenu))
+		{
+			kz_bookmark_menu_remove_menuitems(GTK_MENU_SHELL(submenu), kz);
+			kz_bookmark_menu_append_menuitems(GTK_MENU_SHELL(submenu), kz,
+							  kz->closed_tabs);
+		}
+	}
+        
+	/* update proxy */
 	kz_window_sync_proxy(kz);
 
+	/* for stop/reload button updating */
+	kzembed = KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz));
+	kz_actions_set_sensitive(kz, kzembed);
+
+	/* smart bookmark */
+	kz_actions_update_smartbookmarks(kz, kz_bookmarks->smarts);
+
 	g_free(ui_file);
 	g_free(level);
 }
@@ -750,18 +852,35 @@ void
 kz_window_sync_proxy (KzWindow *kz)
 {
 	gchar path[64] = {0};
-	GtkWidget *widget;
+	GtkWidget *menuitem, *submenu = NULL;
+	GtkAction *action;
+	gboolean use_proxy = FALSE;
 
-	g_snprintf(path, sizeof(path), "/menu/EditMenu%s/ProxyMenu",
+	KZ_CONF_GET("Global", "use_proxy", use_proxy, BOOL);
+	
+	action = gtk_action_group_get_action(kz->actions,
+					     "ToggleProxyUse");
+	if (action)
+		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action),
+					     use_proxy);
+	KZ_WINDOW_SET_VISIBLE(kz, "StockProxyMenu", use_proxy);
+	if (!use_proxy) return;
+
+	g_snprintf(path, sizeof(path), "/menubar/EditMenu%s/ProxyMenu",
 		   kz_ui_level_to_suffix(kz_ui_level()));
-	widget = egg_menu_merge_get_widget(kz->menu_merge, path);
-	if (GTK_IS_MENU_ITEM(widget))
-		widget = gtk_menu_item_get_submenu(GTK_MENU_ITEM(widget));
-	if (GTK_IS_MENU_SHELL(widget))
-	{
-		kz_proxy_menu_remove_menuitems(GTK_MENU_SHELL(widget), kz);
-		kz_proxy_menu_append_menuitems(GTK_MENU_SHELL(widget), kz);
-	}
+	menuitem = gtk_ui_manager_get_widget(kz->menu_merge, path);
+	if (!GTK_IS_MENU_ITEM(menuitem))
+		return;
+	
+	submenu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(menuitem));
+
+	if (!submenu)
+		submenu = gtk_menu_new();
+
+	kz_proxy_menu_remove_menuitems(GTK_MENU_SHELL(submenu), kz);
+	kz_proxy_menu_append_menuitems(GTK_MENU_SHELL(submenu), kz);
+
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);
 }
 
 GtkWidget *
@@ -804,9 +923,6 @@ kz_window_open_new_tab_at_tail (KzWindow *kz, const gchar *url)
 
 	kzembed = KZ_MOZ_EMBED(kz_moz_embed_new(url));
 	
-	/* connect favicon signal */
-	kz_favicon_connect_signal(kz->kzfav, kzembed);
-
 	kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
 
 	gtk_widget_show(GTK_WIDGET(kzembed));
@@ -846,9 +962,6 @@ kz_window_open_new_tab_with_parent (KzWindow *kz, const gchar *url,
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 
 	kzembed = KZ_MOZ_EMBED(kz_moz_embed_new(url));
-	
-	/* connect favicon signal */
-	kz_favicon_connect_signal(kz->kzfav, kzembed);
 
 	kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
 
@@ -938,6 +1051,8 @@ kz_window_open_new_tab_with_parent (KzWindow *kz, const gchar *url,
 	g_signal_emit(G_OBJECT(kz), kz_window_signals[APPEND_TAB_SIGNAL],
 		      0, kzembed, parent);
 
+	g_free(position);
+
 	return GTK_WIDGET(kzembed);
 }
 
@@ -964,9 +1079,6 @@ kz_window_close_tab (KzWindow *kz, GtkWidget *widget)
 
 	}
 
-	/* disconnect favicon signal */
-	kz_favicon_disconnect_signal(kz->kzfav, KZ_MOZ_EMBED(widget));
-
 	ret_page = KZ_CONF_GET_STR("Tab", "page_to_return_when_close");
 	if (ret_page && !strcmp(ret_page, "last_shown"))
 	{
@@ -1202,6 +1314,13 @@ kz_window_destroy (GtkObject *object)
 		g_object_unref(kz->kzfav);
 	kz->kzfav = NULL;
 
+	if (kz->closed_tabs)
+		g_object_unref(kz->closed_tabs);
+	kz->closed_tabs = NULL;
+	if (kz->history_search)
+		g_object_unref(kz->history_search);
+	kz->history_search = NULL;
+
 	g_signal_handlers_disconnect_by_func
 		(G_OBJECT(kz_bookmarks->bookmark_bars),
 		 G_CALLBACK(cb_bookmark_bars_insert_child), kz);
@@ -1212,6 +1331,16 @@ kz_window_destroy (GtkObject *object)
 		(G_OBJECT(kz_bookmarks->bookmark_bars),
 		 G_CALLBACK(cb_bookmark_bars_reordered), kz);
 
+	g_signal_handlers_disconnect_by_func
+		(G_OBJECT(kz_bookmarks->smarts),
+		 G_CALLBACK(cb_smartbookmark_insert_child), kz);
+	g_signal_handlers_disconnect_by_func
+		(G_OBJECT(kz_bookmarks->smarts),
+		 G_CALLBACK(cb_smartbookmark_remove_child), kz);
+	g_signal_handlers_disconnect_by_func
+		(G_OBJECT(kz_bookmarks->smarts),
+		 G_CALLBACK(cb_smartbookmark_reordered), kz);
+
 	g_signal_handlers_disconnect_by_func
 		(G_OBJECT(kz_global_profile),
 		 G_CALLBACK(cb_profile_global_changed), kz);
@@ -1237,7 +1366,7 @@ kz_window_destroy (GtkObject *object)
 static void
 kz_window_set_gesture_items (KzWindow *kz)
 {
-	EggAction *action;
+	GtkAction *action;
 	KzGestureItems *items;
 	GList *list, *node;
 
@@ -1245,14 +1374,14 @@ kz_window_set_gesture_items (KzWindow *kz)
 
 	items = kz_gesture_items_new();
 
-	list = kz_profile_enum_key(kz_global_profile, "Gesture");
+	list = kz_profile_enum_key(kz_global_profile, "Gesture", FALSE);
 
 	for (node = list; node; node = g_list_next(node))
 	{
 		const gchar *action_name = node->data;
 		gchar *gesture;
 
-		action = egg_action_group_get_action(kz->actions,
+		action = gtk_action_group_get_action(kz->actions,
 						     action_name);
 		if (!action) continue;
 
@@ -1278,9 +1407,9 @@ kz_window_set_gesture_items (KzWindow *kz)
 
 
 static void
-cb_window_map (KzWindow *kz, EggAction *action)
+cb_window_map (KzWindow *kz, GtkAction *action)
 {
-	egg_action_activate(action);
+	gtk_action_activate(action);
 	g_signal_handlers_disconnect_by_func
 		(G_OBJECT(kz), G_CALLBACK(cb_window_map), action);
 }
@@ -1289,7 +1418,7 @@ cb_window_map (KzWindow *kz, EggAction *action)
 static void
 kz_window_store_session (KzWindow *kz)
 {
-	EggAction *action;
+	GtkAction *action;
 	gboolean save_session = FALSE;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
@@ -1297,9 +1426,9 @@ kz_window_store_session (KzWindow *kz)
 	KZ_CONF_GET("Session", "save", save_session, BOOL);
 	if (save_session && g_list_length(window_list) == 1)
 	{
-		action = egg_action_group_get_action(kz->actions,
+		action = gtk_action_group_get_action(kz->actions,
 						     "SaveSession");
-		egg_action_activate(action);
+		gtk_action_activate(action);
 	}
 }
 
@@ -1307,7 +1436,7 @@ kz_window_store_session (KzWindow *kz)
 static void
 kz_window_restore_session (KzWindow *kz)
 {
-	EggAction *action;
+	GtkAction *action;
 	gboolean restore_session = FALSE;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
@@ -1316,10 +1445,10 @@ kz_window_restore_session (KzWindow *kz)
 		    restore_session, BOOL);
 	if (restore_session && g_list_length(window_list) == 1)
 	{
-		action = egg_action_group_get_action(kz->actions,
+		action = gtk_action_group_get_action(kz->actions,
 						     "RestoreSession");
 		if (GTK_WIDGET_MAPPED(kz))
-			egg_action_activate(action);
+			gtk_action_activate(action);
 		else
 			g_signal_connect(G_OBJECT(kz), "map",
 					 G_CALLBACK(cb_window_map), action);
@@ -1330,7 +1459,7 @@ kz_window_restore_session (KzWindow *kz)
 void
 kz_window_store_state (KzWindow *kz)
 {
-	EggAction *action;
+	GtkAction *action;
 	gint i, client_x, client_y, width, height;
 	const gchar *label;
 	gboolean active;
@@ -1353,9 +1482,9 @@ kz_window_store_state (KzWindow *kz)
 	 */
 	for (i = 0; i < G_N_ELEMENTS(positems); i++)
 	{
-		action = egg_action_group_get_action(kz->actions,
+		action = gtk_action_group_get_action(kz->actions,
 						     positems[i].sidebar_act);
-		active = egg_toggle_action_get_active(EGG_TOGGLE_ACTION(action));
+		active = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
 		if (active)
 		{
 			KZ_CONF_SET_STR("MainWindow", "sidebar_pos",
@@ -1388,9 +1517,9 @@ kz_window_store_state (KzWindow *kz)
 		KZ_CONF_SET_STR("MainWindow", "sidebar", label);
 
 	/* visible */
-	action = egg_action_group_get_action(kz->actions,
+	action = gtk_action_group_get_action(kz->actions,
 					     "ShowHideSidebar");
-	active = egg_toggle_action_get_active(EGG_TOGGLE_ACTION(action));
+	active = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
 	KZ_CONF_SET("MainWindow", "show_sidebar", active, BOOL);
 
 
@@ -1400,9 +1529,9 @@ kz_window_store_state (KzWindow *kz)
 
 	for (i = 0; i < G_N_ELEMENTS(positems); i++)
 	{
-		action = egg_action_group_get_action(kz->actions,
+		action = gtk_action_group_get_action(kz->actions,
 						     positems[i].tab_act);
-		active = egg_toggle_action_get_active(EGG_TOGGLE_ACTION(action));
+		active = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
 		if (active)
 		{
 			KZ_CONF_SET_STR("MainWindow", "tab_pos",
@@ -1423,7 +1552,7 @@ kz_window_store_state (KzWindow *kz)
 void
 kz_window_restore_state (KzWindow *kz)
 {
-	EggAction *action = NULL;
+	GtkAction *action = NULL;
 	gint width = 640, height = 450, sidebar_width = 150;
 	gboolean active = FALSE;
 	gchar *tab_pos, *label, *sidebar_pos;
@@ -1431,7 +1560,6 @@ kz_window_restore_state (KzWindow *kz)
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-
 	/*
 	 * window size
 	 */
@@ -1441,7 +1569,6 @@ kz_window_restore_state (KzWindow *kz)
 
 	gtk_window_set_default_size(GTK_WINDOW(kz), width, height);
 
-
 	/*
 	 * sidebar
 	 */
@@ -1453,9 +1580,9 @@ kz_window_restore_state (KzWindow *kz)
 
 	/* visible */
 	KZ_CONF_GET("MainWindow", "show_sidebar", active, BOOL);
-	action = egg_action_group_get_action(kz->actions,
+	action = gtk_action_group_get_action(kz->actions,
 					     "ShowHideSidebar");
-	egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action), active);
+	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), active);
 	if (active)
 	{
 	sidebar_pos = KZ_CONF_GET_STR("MainWindow", "sidebar_pos");
@@ -1465,9 +1592,9 @@ kz_window_restore_state (KzWindow *kz)
 
 		/* width */
 		KZ_CONF_GET("MainWindow", "sidebar_width", sidebar_width, INT);
-		action = egg_action_group_get_action(kz->actions,
+		action = gtk_action_group_get_action(kz->actions,
 						     positems[i].sidebar_act);
-		egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action),
+		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action),
 					     TRUE);
 		/*
 		 * On startup time, allocation.width and allocation.height are not set
@@ -1487,9 +1614,9 @@ kz_window_restore_state (KzWindow *kz)
 	{
 		if (strcasecmp(tab_pos, positems[i].pos)) continue;
 
-		action = egg_action_group_get_action(kz->actions,
+		action = gtk_action_group_get_action(kz->actions,
 						     positems[i].tab_act);
-		egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action),
+		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action),
 					     TRUE);
 	}
 	g_free(tab_pos);
@@ -1747,6 +1874,10 @@ cb_profile_global_changed (KzProfile *profile,
 		{
 			kz_window_sync_ui_level(kz);
 		}
+		if (strcmp(key, "use_proxy") == 0)
+		{
+			kz_window_sync_proxy(kz);
+		}
 		break;
 	case 'p':
 		if (strcmp(key, "proxy_name") == 0)
@@ -1761,9 +1892,9 @@ cb_profile_global_changed (KzProfile *profile,
 
 static void
 cb_profile_proxy_changed (KzProfile *profile,
-			   const gchar *section, const gchar *key,
-			   const gchar *old_value,
-			   KzWindow *kz)
+			  const gchar *section, const gchar *key,
+			  const gchar *old_value,
+			  KzWindow *kz)
 {
 	g_return_if_fail(KZ_IS_PROFILE(profile));
 	g_return_if_fail(KZ_IS_WINDOW(kz));
@@ -1897,10 +2028,45 @@ cb_bookmark_bars_reordered (KzBookmark *bookmark, KzWindow *kz)
 
 
 static void
-cb_menu_merge_add_widget (EggMenuMerge *merge, GtkWidget *widget, GtkBox *box)
+cb_smartbookmark_insert_child (KzBookmark *bookmark,
+			       KzBookmark *child, KzBookmark *sibling,
+			       KzWindow *kz)
+{
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+	g_return_if_fail(KZ_IS_BOOKMARK(child));
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	kz_actions_insert_smartbookmark(kz, bookmark, child, sibling);
+}
+
+
+static void
+cb_smartbookmark_remove_child (KzBookmark *bookmark, KzBookmark *child,
+			       KzWindow *kz)
+{
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+	g_return_if_fail(KZ_IS_BOOKMARK(child));
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	kz_actions_remove_smartbookmark(kz, bookmark, child);	
+}
+
+
+static void
+cb_smartbookmark_reordered (KzBookmark *bookmark, KzWindow *kz)
+{
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	kz_actions_update_smartbookmarks(kz, bookmark);	
+}
+
+
+
+static void
+cb_menu_merge_add_widget (GtkUIManager *merge, GtkWidget *widget, GtkBox *box)
 {
 	gtk_box_pack_start (box, widget, FALSE, FALSE, 0);
-	gtk_widget_show (widget);
 }
 
 
@@ -1908,12 +2074,12 @@ static gboolean
 idle_focus_location_entry(gpointer data)
 {
 	KzWindow *kz = KZ_WINDOW(data);
-	EggAction *action;
+	GtkAction *action;
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
-	action = egg_action_group_get_action(kz->actions,
+	action = gtk_action_group_get_action(kz->actions,
 					     "FocusLocationEntry");
 	if(action)
-		egg_action_activate(action);
+		gtk_action_activate(action);
 	return FALSE;
 }
 
@@ -1926,7 +2092,7 @@ cb_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 	KzMozEmbed *cur = KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz));
 	gchar *title;
 	const gchar *location;
-	EggAction *action;
+	GtkAction *action;
 	KzTabLabel *tab;
 	gboolean focus;
 
@@ -1939,26 +2105,36 @@ cb_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 	title = kz_moz_embed_ensure_title(kzembed);
 	location = kz_moz_embed_get_location(kzembed);
 	if (title)
+	{
 		gtk_window_set_title(GTK_WINDOW(kz), title);
+		g_free(title);
+	}
 	else
+	{
 		gtk_window_set_title(GTK_WINDOW(kz), _("Kazehakase"));
-
-	g_free(title);
-	title = NULL;
-
-	action = egg_action_group_get_action(kz->actions, "LocationEntry");
-
-	if (EGG_IS_ENTRY_ACTION(action))
+	}
+	
+	action = gtk_action_group_get_action(kz->actions, "LocationEntry");
+	if (KZ_IS_ENTRY_ACTION(action))
 	{
 		if (location)
-			egg_entry_action_set_text(EGG_ENTRY_ACTION(action),
+			kz_entry_action_set_text(KZ_ENTRY_ACTION(action),
 						  location);
 		else
-			egg_entry_action_set_text(EGG_ENTRY_ACTION(action), "");
+			kz_entry_action_set_text(KZ_ENTRY_ACTION(action), "");
+	}
+	
+	action = gtk_action_group_get_action(kz->actions, "Zoom");
+	if (KZ_IS_ZOOM_ACTION(action))
+	{
+		gint ratio;
+		ratio = kz_moz_embed_zoom_get(kzembed);
+		kz_zoom_action_set_ratio(KZ_ZOOM_ACTION(action), ratio);
 	}
 
-	tab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(notebook, GTK_WIDGET(kzembed)));
-	g_return_if_fail(KZ_IS_TAB_LABEL(tab));
+	tab = KZ_TAB_LABEL(kz_window_get_tab_label(kz, GTK_WIDGET(kzembed)));
+	g_return_if_fail(tab);
+
 	if(kz_tab_label_get_state(tab) == KZ_TAB_LABEL_STATE_LOADED)
 	{
 		kz_tab_label_set_state(tab, KZ_TAB_LABEL_STATE_NORMAL);
@@ -1971,6 +2147,7 @@ cb_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 	}
 
 	kz_actions_set_sensitive(kz, kzembed);
+	kz_actions_set_tab_sensitive(kz, kzembed);
 
 	kz->priv->view_hist = g_list_remove(kz->priv->view_hist, kzembed);
 	kz->priv->view_hist = g_list_prepend(kz->priv->view_hist, kzembed);
@@ -2014,10 +2191,13 @@ cb_find_key_release (GtkWidget *widget, GdkEventKey *event, KzWindow *kz)
 	}
 	else
 	{
-		gboolean back;
-		back = gtk_toggle_button_get_active(toggle);
-		kz->priv->did_find
-			= kz_moz_embed_incremental_search(embed, text, back);
+		if (!(event->state & GDK_SHIFT_MASK))
+		{
+			gboolean back;
+			back = gtk_toggle_button_get_active(toggle);
+			kz->priv->did_find
+				= kz_moz_embed_incremental_search(embed, text, back);
+		}
 	}
 
 	return FALSE;
@@ -2099,22 +2279,22 @@ cb_notebook_scroll_event (GtkWidget *widget, GdkEventScroll *event,
 
 
 static void
-cb_sidebar_map (GtkWidget *widget, EggToggleAction *action)
+cb_sidebar_map (GtkWidget *widget, GtkToggleAction *action)
 {
-	g_return_if_fail(EGG_IS_TOGGLE_ACTION(action));
+	g_return_if_fail(GTK_IS_TOGGLE_ACTION(action));
 
 	KZ_SIDEBAR(widget)->kz->priv->sidebar_was_shown = TRUE;
 
-	egg_toggle_action_set_active(action, TRUE);
+	gtk_toggle_action_set_active(action, TRUE);
 }
 
 
 static void
-cb_sidebar_unmap (GtkWidget *widget, EggToggleAction *action)
+cb_sidebar_unmap (GtkWidget *widget, GtkToggleAction *action)
 {
-	g_return_if_fail(EGG_IS_TOGGLE_ACTION(action));
+	g_return_if_fail(GTK_IS_TOGGLE_ACTION(action));
 
-	egg_toggle_action_set_active(action, FALSE);
+	gtk_toggle_action_set_active(action, FALSE);
 }
 
 
@@ -2149,19 +2329,19 @@ cb_embed_location_changed (GtkMozEmbed *embed, KzWindow *kz)
 	newLocation = kz_moz_embed_get_location(KZ_MOZ_EMBED(embed));
 	if (newLocation)
 	{
-		EggAction *action
-			= egg_action_group_get_action(kz->actions,
+		GtkAction *action
+			= gtk_action_group_get_action(kz->actions,
 						      "LocationEntry");
 		if (embed == GTK_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz))
-		    && EGG_IS_ENTRY_ACTION(action))
+		    && KZ_IS_ENTRY_ACTION(action))
 		{
-			egg_entry_action_set_text(EGG_ENTRY_ACTION(action),
-						  newLocation);
+			kz_entry_action_set_text(KZ_ENTRY_ACTION(action),
+						 newLocation);
 		}
 	}
 	else
 	{
-		g_print("failed to get location!\n");
+		/* g_print("failed to get location!\n"); */
 	}
 
 	kz_actions_set_sensitive(kz, KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
@@ -2174,7 +2354,7 @@ cb_embed_link_message (GtkMozEmbed *embed, KzWindow *kz)
 	gchar *newMessage;
 
 	g_return_if_fail(KZ_IS_MOZ_EMBED(embed));
-	g_return_if_fail(KZ_WINDOW(kz));
+	g_return_if_fail(KZ_IS_WINDOW(kz));
 
 	newMessage = kz_moz_embed_get_link_message(KZ_MOZ_EMBED(embed));
 	if(kz->statusbar)
@@ -2336,33 +2516,9 @@ cb_embed_dom_key_down (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 static gint
 cb_embed_dom_key_up (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 {
-	gboolean typeahead = FALSE;
-
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(embed), FALSE);
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
 
-	KZ_CONF_GET("Mozilla", "typeahead", typeahead, BOOL);
-
-	if (!typeahead)
-	{
-		KzMozEmbed *kzembed;
-		gulong key_code;
-		glong type;
-		KzEmbedEventKey *info;
-
-		kzembed = KZ_MOZ_EMBED(embed);
-		
-		type = kz_moz_embed_get_key_event_info(kzembed, event, &info);
-		key_code = info->key;
-
-		if ((key_code == 191) && !(type & KZ_CONTEXT_INPUT)) /* '/' */
-		{
-			gtk_widget_grab_focus(kz->find_area);
-		}
-
-		kz_embed_event_free((KzEmbedEvent *) info);
-	}
-
 	kz_actions_set_sensitive(kz, KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
 
 	return FALSE;
@@ -2393,6 +2549,8 @@ cb_embed_dom_mouse_click (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 	case 0:
 		if ((type & KZ_CONTEXT_LINK) && info->cinfo.link)
 		{
+			const gchar *location;
+
 			kztab = KZ_TAB_LABEL
 				(gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook),
 							    widget));
@@ -2406,6 +2564,12 @@ cb_embed_dom_mouse_click (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 				kz_embed_event_free((KzEmbedEvent *) info);
 				return TRUE;
 			}
+			location = kz_moz_embed_get_location(KZ_MOZ_EMBED(embed));
+			if (!strncmp(location, "history-search:", 15))
+			{
+				kz_moz_embed_load_url(KZ_MOZ_EMBED(embed), 
+						      info->cinfo.link);
+			}
 		}
 		break;
 	case 1:
@@ -2416,16 +2580,15 @@ cb_embed_dom_mouse_click (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 		}
 		else if (!(type & KZ_CONTEXT_INPUT))
 		{
-			GtkClipboard *clipboard;
-			gchar *text;
+			GtkAction *action;
+
+			action = gtk_action_group_get_action(kz->actions,
+							     "OpenSelection");
 
-			clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
-			text = gtk_clipboard_wait_for_text(clipboard);
 			/* what about uri list? */
-			if (text)
-				kz_moz_embed_load_url(KZ_MOZ_EMBED(embed),
-						      text);
-			g_free(text);
+			if (action)
+				gtk_action_activate(action);
+
 			break;
 		}
 		break;
@@ -2447,7 +2610,7 @@ cb_embed_dom_mouse_dbl_click (GtkMozEmbed *embed, gpointer event, KzWindow *kz)
 	gint button;
 
 	g_return_val_if_fail(GTK_IS_MOZ_EMBED(embed), FALSE);
-	g_return_val_if_fail(KZ_WINDOW(kz), FALSE);
+	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
 
      	type = kz_moz_embed_get_mouse_event_info(embed, event, &info);	
 	button = info->button;
@@ -2659,6 +2822,8 @@ cb_notebook_drag_data_received (GtkWidget *widget,
 					      n - 1);
 			kz_actions_set_sensitive
 				(kz, KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+			kz_actions_set_tab_sensitive
+				(kz, KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
 		}
 		else
 		{
@@ -2714,3 +2879,16 @@ cb_focus_out_event            (GtkWidget       *widget,
 	}
 	return FALSE;
 }
+
+void
+kz_window_create_thumbnail_creator (KzWindow *kz)
+{
+	kz->thumb_creator = kz_moz_thumbnail_creator_new();
+	gtk_widget_set_size_request(GTK_WIDGET(kz->thumb_creator), 0, 0);
+	gtk_widget_show(GTK_WIDGET(kz->thumb_creator));
+		
+	gtk_box_pack_start(GTK_BOX(kz->statusbar_area),
+			   GTK_WIDGET(kz->thumb_creator),
+			   FALSE, FALSE, 0);
+}
+
