@@ -28,6 +28,10 @@
 #include "kz-gesture.h"
 #include "kz-embed-event.h"
 #include "kz-favicon.h"
+#include "kz-mozembed.h"
+#include "kz-bookmark.h"
+
+#include "kz-mozthumbnailcreator.h"
 
 G_BEGIN_DECLS
 
@@ -71,10 +75,17 @@ struct _KzWindow
 	GtkActionGroup *tabpop_actions;
 	GtkUIManager   *menu_merge;
 
+	/* bookmarks */
+	KzBookmark *closed_tabs;
+	KzBookmark *history_search;
+
 	/* handle for favicon */
 	KzFavicon *kzfav;
 
 	KzWindowPriv *priv;
+	
+	/* for creating thumbnails */
+	KzMozThumbnailCreator *thumb_creator;
 };
 
 struct _KzWindowClass
@@ -126,6 +137,7 @@ GNode            *kz_window_get_tree                 (KzWindow    *kz);
 const KzEmbedEventMouse *
 		  kz_window_get_mouse_event_info     (KzWindow    *kz);
 
+void		  kz_window_create_thumbnail_creator (KzWindow    *kz);
 
 
 #define KZ_WINDOW_NTH_PAGE(kz, n) \
