@@ -36,6 +36,7 @@
 #include "intl.h"
 #include "locale.h"
 #include "utils.h"
+#include "estsearch.h"
 #include "mozilla.h"
 
 #ifdef USE_GTK_2_2
@@ -57,6 +58,7 @@ N_("Usage: kazehakase [OPTION...] [URI...]\n"
 );
 
 gboolean exists_estindex = FALSE;
+gchar   *estversion;
 
 /******************************************************************************
  *                                                                            *
@@ -407,6 +409,47 @@ check_kazehakase_process (int argc, char *argv[])
 }
 
 
+static void
+clean_history_cache (void)
+{
+	gchar *history_dir;
+	guint limit_days = 30;
+	gboolean limit = FALSE;
+	time_t limit_seconds;
+	
+	KZ_CONF_GET("History", "limit_days", limit_days, INT);
+	KZ_CONF_GET("History", "limit_cache", limit, BOOL);
+	if (limit)
+	{
+		GSList *purge_list = NULL;
+		gchar **purge_files = NULL;
+
+		limit_seconds = limit_days * 86400;
+		history_dir = g_build_filename(g_get_home_dir(),
+					       HISTORY_DIR,
+					       NULL); 
+		purge_list = purge_history_file(history_dir, limit_seconds);
+		g_free(history_dir);
+
+		if (purge_list)
+		{
+			GSList *slist;
+			gint n = g_slist_length(purge_list);
+			purge_files = g_new0 (gchar*, n + 1);
+
+			purge_files[n--] = NULL;
+			for (slist = purge_list; slist; slist = slist->next)
+				purge_files[n--] = slist->data;
+
+			g_slist_free(purge_list);
+			/* optimize and purge history index */
+			if (exists_estindex)
+				estsearch_purge_index(purge_files);
+		 
+			g_strfreev(purge_files);
+		}
+	}
+}
 /******************************************************************************
  *                                                                            *
  *                                  Main                                      *
@@ -417,7 +460,7 @@ open_app_info (void)
 {
 	gchar *sysconf_file, *conf_file, *session_file;
 	gchar *bookmark_file, *sys_bookmark_file;
-	gchar *scrap_file;
+	gchar *clip_file;
 	gchar *bookmark_bar_file, *sys_bookmark_bar_file;
 	gchar *smartbookmark_file, *sys_smartbookmark_file;
 	gchar *accel_prefs_file;
@@ -426,41 +469,48 @@ open_app_info (void)
 
 	/* load prefs */
 	sysconf_file = g_strdup(KZ_SYSCONFDIR "/kzrc");
-	conf_file = g_strconcat(g_get_home_dir(), "/.kazehakase/kzrc", NULL);
+	conf_file = g_build_filename(g_get_home_dir(), ".kazehakase",
+				     "kzrc", NULL);
+	
 	kz_global_profile = kz_profile_open(conf_file, sysconf_file);
 
 	/* load sessions */
-	session_file = g_strconcat(g_get_home_dir(),
-				   "/.kazehakase/sessionrc",
-				   NULL);
+	session_file = g_build_filename(g_get_home_dir(),
+				   	".kazehakase",
+					"sessionrc",
+				   	NULL);
 	kz_session = kz_profile_open(session_file, NULL);
 
 	/* load bookmarks */
-	bookmark_file         = g_strconcat(g_get_home_dir(),
-					    "/.kazehakase/bookmarks.xml",
-					    NULL);
-	sys_bookmark_file     = g_strconcat(KZ_SYSCONFDIR,
-					    "/bookmarks.xml",
-					    NULL);
+	bookmark_file         = g_build_filename(g_get_home_dir(),
+					    	 ".kazehakase",
+					    	 "bookmarks.xml",
+					    	 NULL);
+	sys_bookmark_file     = g_build_filename(KZ_SYSCONFDIR,
+					    	 "bookmarks.xml",
+					    	 NULL);
 
-        bookmark_bar_file     = g_strconcat(g_get_home_dir(),
-					    "/.kazehakase/bookmarkbar.xml",
-					    NULL);
-	sys_bookmark_bar_file = g_strconcat(KZ_SYSCONFDIR,
-					    "/bookmarkbar.xml",
-					    NULL);
-	scrap_file            = g_strconcat(g_get_home_dir(),
-					    "/.kazehakase/scrap.html",
-					    NULL);
-        smartbookmark_file    = g_strconcat(g_get_home_dir(),
-					    "/.kazehakase/smartbookmarks.xml",
-					    NULL);
-	sys_smartbookmark_file= g_strconcat(KZ_SYSCONFDIR,
-					    "/smartbookmarks.xml",
-					    NULL);
+        bookmark_bar_file     = g_build_filename(g_get_home_dir(),
+					    	 ".kazehakase",
+						 "bookmarkbar.xml",
+					    	 NULL);
+	sys_bookmark_bar_file = g_build_filename(KZ_SYSCONFDIR,
+					    	 "bookmarkbar.xml",
+					    	 NULL);
+	clip_file             = g_build_filename(g_get_home_dir(),
+					    	 ".kazehakase",
+						 "clip.xml",
+					    	 NULL);
+        smartbookmark_file    = g_build_filename(g_get_home_dir(),
+					    	 ".kazehakase",
+					    	 "smartbookmarks.xml",
+					    	 NULL);
+	sys_smartbookmark_file= g_build_filename(KZ_SYSCONFDIR,
+					    	 "smartbookmarks.xml",
+					    	 NULL);
 	
 	kz_bookmarks = kz_root_bookmark_new(bookmark_file, sys_bookmark_file,
-					    scrap_file, NULL);
+					    clip_file, NULL);
 
 	kz_root_bookmark_add_smartbookmark_file(kz_bookmarks,
 						smartbookmark_file,
@@ -471,13 +521,16 @@ open_app_info (void)
 
 
 	/* Load other prefs... */
-	accel_prefs_file = g_strconcat (g_get_home_dir(),
-					"/.kazehakase/keyaccelrc",
-					NULL);
+	accel_prefs_file = g_build_filename (g_get_home_dir(),
+					     ".kazehakase",
+					     "keyaccelrc",
+					     NULL);
 	gtk_accel_map_load (accel_prefs_file);
 
-	sysproxy_file = g_strdup(KZ_SYSCONFDIR "/proxyrc");
-	proxy_file = g_strconcat (g_get_home_dir(), "/.kazehakase/proxyrc", NULL);
+	sysproxy_file = g_build_filename(KZ_SYSCONFDIR, "proxyrc", NULL);
+	proxy_file = g_build_filename (g_get_home_dir(),
+				       ".kazehakase",
+				       "proxyrc", NULL);
 	kz_proxy = kz_profile_open(proxy_file, sysproxy_file);
 
 	kz_icons_init();
@@ -490,12 +543,19 @@ open_app_info (void)
 	{
 		exists_estindex = TRUE;
 		g_free(estindex);
+		/* check estraier version */
+		estversion = estsearch_get_version();
 	}
+
+	/* make dir for thumbnails */
+	make_thumbnails_dir();
+
 	g_free(sysconf_file);
 	g_free(conf_file);
 	g_free(session_file);
 	g_free(bookmark_file);
-	g_free(scrap_file);
+	g_free(sys_bookmark_file);
+	g_free(clip_file);
 	g_free(bookmark_bar_file);
 	g_free(sys_bookmark_bar_file);
 	g_free(smartbookmark_file);
@@ -511,15 +571,19 @@ close_app_info (void)
 {
 	gchar *accel_prefs_file;
 
-	accel_prefs_file = g_strconcat (g_get_home_dir(),
-					"/.kazehakase/keyaccelrc",
-					NULL);
+	accel_prefs_file = g_build_filename (g_get_home_dir(),
+					     ".kazehakase",
+					     "keyaccelrc",
+					     NULL);
 
 	gtk_accel_map_save(accel_prefs_file);
 	g_free(accel_prefs_file);
 
 	mozilla_exit();
 
+	/* clean history cache */
+	clean_history_cache();
+
 	kz_root_bookmark_save_all(kz_bookmarks);
 	g_object_unref(G_OBJECT(kz_bookmarks));
 
@@ -527,6 +591,12 @@ close_app_info (void)
 	kz_profile_close(kz_session);
 	kz_profile_close(kz_proxy);
 
+	if (estversion)
+	{
+		g_free(estversion);
+		estversion = NULL;
+	}
+	
 #if USE_MIGEMO
 	if (migemo)
 	{
@@ -546,7 +616,6 @@ main (int argc, char *argv[])
 {
 	KzWindow *kz = NULL;
 	GtkWidget *dupl_check_server = NULL;
-	gchar *uri = NULL;
 	gint i;
 
 	/* initialize */
@@ -572,7 +641,7 @@ main (int argc, char *argv[])
 			return 0;
 		}
 	}
-	
+
 #ifdef USE_GTK_2_2
 	gtk24backports_init();
 #endif
@@ -588,19 +657,52 @@ main (int argc, char *argv[])
 	open_app_info();
 
 	/* create window */	
-	if (argc > 1)
-		uri = complement_scheme(argv[1]);
-	kz = KZ_WINDOW(kz_window_new(uri));
-	if (uri)
-		g_free(uri);
+	kz = KZ_WINDOW(kz_window_new(NULL));
 
-	for (i = 2; i < argc; i++)
+       	/* 
+	 * gtk_window_pargse_geometry() needs to be called 
+	 * before gtk_widget_show().
+	 */ 
+	for (i = 1; i < argc; i++)
 	{
+		if (!strcmp(argv[i], "-geometry") &&
+		     *argv[i+1] && argv[i+1])
+		{
+			gtk_window_parse_geometry(GTK_WINDOW(kz),
+						  argv[i+1]);
+			i++;
+			continue;
+		}
+	}
+
+	gtk_widget_show(GTK_WIDGET(kz));
+	
+	/* process arguments */
+	for (i = 1; i < argc; i++)
+	{
+		gchar *uri;
+		if ((!strcmp(argv[i], "--action") ||
+		     !strcmp(argv[i], "-a")) &&
+		    *argv[i+1] &&
+		    argv[i+1])
+		{
+			GList* list = kz_window_get_window_list();
+			activate_action(list, argv[i+1]);
+			i++;
+			continue;
+		}
+		else if (!strcmp(argv[i], "-geometry") &&
+		         *argv[i+1] && argv[i+1])
+		{
+			i++;
+			continue;
+		}
+
+		/* open URL */
 		uri = complement_scheme(argv[i]);
 		kz_window_open_new_tab(kz, uri);
 		g_free(uri);
 	}
-	gtk_widget_show(GTK_WIDGET(kz));
 
 	/* event loop */
 	gtk_main();
