@@ -22,12 +22,15 @@
 #include "kz-mozembed.h"
 
 #include <string.h>
+#include <math.h>
 #include <gtkmozembed.h>
 #include <gtkmozembed_internal.h>
 
 #include "kazehakase.h"
+#include "kz-window.h"
 #include "kz-mozwrapper.h"
 #include "kz-mozprogresslistener.h"
+#include "kz-mozthumbnailcreator.h"
 #include "gobject-utils.h"
 #include "intl.h"
 #include "mozilla.h"
@@ -37,6 +40,7 @@
 #include "migemo.h"
 #endif
 #include "utils.h"
+#include "estsearch.h"
 
 #include <nsCOMPtr.h>
 #include <nsIDOMDocument.h>
@@ -59,6 +63,7 @@
 #include <docshell/nsIDocShell.h>
 #include <docshell/nsIDocShellTreeItem.h>
 #include <docshell/nsIDocShellTreeOwner.h>
+#include <nsIPresShell.h>
 #include <nsIDOMNodeList.h>
 #include <nsIDOMWindow.h>
 #include <nsISelection.h>
@@ -78,8 +83,16 @@
 #include <nsCWebBrowserPersist.h>
 #include <widget/nsIBaseWindow.h>
 #include <nsIWebPageDescriptor.h>
-#include <nsIPresContext.h>
 #include <nsIEventStateManager.h>
+#include <nsICommandManager.h>
+#include <nsTime.h>
+
+#define N_LINKS 6
+
+static gchar *nav_names[N_LINKS] =
+{
+ "NEXT", "PREV", "INDEX", "CONTENTS", "START", "ALTERNATE"
+};
 
 struct _KzMozEmbedPriv
 {
@@ -91,18 +104,12 @@ struct _KzMozEmbedPriv
 	gboolean lock;
 
 	/* for navigation link */
-	gchar *link_next;
-	gchar *link_prev;
-	gchar *link_index;
-	gchar *link_contents;
-	gchar *link_start;
-	
-	/* for alternate rss */
-	gchar *link_rss;
-#if USE_MIGEMO
-	/* for migemo */
+	gchar *nav_links[N_LINKS];
+
+#ifdef USE_MIGEMO
 	gchar *migemo_keyword;
 #endif
+	gchar *first_url;
 };
 
 enum {
@@ -133,7 +140,11 @@ static void kz_moz_embed_net_state_all (GtkMozEmbed *embed,
 					const char  *aURI,
 					gint         state,
 					guint        status);
-
+#if 0
+static void kz_moz_embed_security_change (GtkMozEmbed *embed,
+					  gpointer request,
+					  guint state);
+#endif
 static void kz_moz_embed_get_head_link (KzMozEmbed *kzembed);
 
 static void kz_moz_embed_navigation_link_free(KzMozEmbed *kzembed);
@@ -142,19 +153,11 @@ static gint kz_moz_embed_dom_key_down  (GtkMozEmbed *embed, gpointer event);
 static gint kz_moz_embed_dom_key_up    (GtkMozEmbed *embed, gpointer event);
 static gint kz_moz_embed_dom_key_press (GtkMozEmbed *embed, gpointer event);
 
-static gchar    *mozilla_get_attribute      (nsIDOMNode *node,
-					     gchar *attribute);
-static gboolean  mozilla_get_link_from_node (nsIDOMDocument *domDoc,
-					     nsIDOMNode *node,
-					     gchar **url,
-					     gchar **inner_html);
 static glong     mozilla_set_event_context  (KzMozEmbed *kzembed,
 					     nsIDOMEventTarget *target,
 					     KzEmbedEvent *info);
 
-static gboolean  mozilla_get_head_link_from_node (nsIDOMDocument *domDoc,
-						  nsIDOMNode *node,
-						  gchar **url);
+static gchar	*mozilla_store_history_file (KzMozEmbed *kzembed);
 
 static GtkMozEmbedClass *parent_class = NULL;
 
@@ -194,6 +197,7 @@ kz_moz_embed_class_init (KzMozEmbedClass *klass)
 	// GtkMozEmbedSignals
 	moz_embed_class->net_state_all   = kz_moz_embed_net_state_all;
 #if 0
+	moz_embed_class->security_change = kz_moz_embed_security_change;
 	moz_embed_class->dom_mouse_over  = kz_moz_embed_dom_mouse_over;
 	moz_embed_class->dom_mouse_out   = kz_moz_embed_dom_mouse_out;
 	moz_embed_class->net_state       = kz_moz_embed_net_state;
@@ -237,10 +241,10 @@ kz_moz_embed_class_init (KzMozEmbedClass *klass)
 				G_TYPE_NONE, 0);
 }
 
-
 static void
 kz_moz_embed_init (KzMozEmbed *kzembed)
 {
+	gint i;
 	// widgets
 	kzembed->popup_window          = NULL;
 
@@ -260,15 +264,16 @@ kz_moz_embed_init (KzMozEmbed *kzembed)
 	kzembed->priv->size_inited    = FALSE;
 	kzembed->priv->total_requests = 0;
 	kzembed->priv->cur_requests   = 0;
-	kzembed->priv->link_index     = NULL;
-	kzembed->priv->link_contents  = NULL;
-	kzembed->priv->link_start     = NULL;
-	kzembed->priv->link_next      = NULL;
-	kzembed->priv->link_prev      = NULL;
-	kzembed->priv->link_rss       = NULL;
-#if USE_MIGEMO
+	
+	for (i =0; i < N_LINKS; i++)
+	{
+		kzembed->priv->nav_links[i] = NULL;
+	}
+#ifdef USE_MIGEMO
 	kzembed->priv->migemo_keyword = NULL;
 #endif
+	kzembed->priv->first_url      = NULL;
+	kz_moz_embed_load_url(kzembed, "about:blank");
 }
 
 GtkWidget *
@@ -283,7 +288,12 @@ kz_moz_embed_new (const gchar *url)
 	g_moz_embed_set_chrome_mask(GTK_MOZ_EMBED(kzembed->mozembed),
 				    actualChromeMask);
 #endif
-	kz_moz_embed_load_url(kzembed, url);
+	
+	// To use the gesture in empty tab, 
+	// After "about:balnk" was loaded, the first url start to load.  
+	if (url)
+		kzembed->priv->first_url = g_strdup(url);
+	//kz_moz_embed_load_url(kzembed, url);
 
 	return GTK_WIDGET(kzembed);
 }
@@ -299,13 +309,11 @@ kz_moz_embed_destroy (GtkObject *object)
 	kzembed->location = NULL;
 	g_free(kzembed->title);
 	kzembed->title = NULL;
-#if USE_MIGEMO
-	if (kzembed->priv->migemo_keyword)
-	{
-		g_free(kzembed->priv->migemo_keyword);
-		kzembed->priv->migemo_keyword = NULL;
-	}
-#endif
+
+	if (kzembed->priv->first_url)
+		g_free(kzembed->priv->first_url);
+	kzembed->priv->first_url = NULL;
+
 	if (GTK_OBJECT_CLASS(parent_class)->destroy)
 		GTK_OBJECT_CLASS(parent_class)->destroy(object);
 }
@@ -324,7 +332,7 @@ kz_moz_embed_realize (GtkWidget *widget)
 
 	if (!kzembed->priv->wrapper)
 	{
-		kzembed->priv->wrapper = new KzMozWrapper;
+		kzembed->priv->wrapper = new KzMozWrapper();
 		nsresult rv = kzembed->priv->wrapper->Init(kzembed);
 		if (NS_FAILED(rv))
 		{
@@ -358,25 +366,21 @@ kz_moz_embed_unrealize (GtkWidget *widget)
 static void
 kz_moz_embed_navigation_link_free(KzMozEmbed *kzembed)
 {	
-	if (kzembed->priv->link_index)
-		g_free(kzembed->priv->link_index);
-	if (kzembed->priv->link_contents)
-		g_free(kzembed->priv->link_contents);
-	if (kzembed->priv->link_start)
-		g_free(kzembed->priv->link_start);
-	if (kzembed->priv->link_prev)
-		g_free(kzembed->priv->link_prev);
-	if (kzembed->priv->link_next)
-		g_free(kzembed->priv->link_next);
-	if (kzembed->priv->link_rss)
-		g_free(kzembed->priv->link_rss);
+	gint i;
 	
-	kzembed->priv->link_index     = NULL;
-	kzembed->priv->link_contents  = NULL;
-	kzembed->priv->link_start     = NULL;
-	kzembed->priv->link_prev      = NULL;
-	kzembed->priv->link_next      = NULL;
-	kzembed->priv->link_rss       = NULL;
+	for (i =0; i < N_LINKS; i++)
+	{
+		if (kzembed->priv->nav_links[i])
+		{
+			g_free(kzembed->priv->nav_links[i]);
+			kzembed->priv->nav_links[i] = NULL;
+		}
+	}
+#ifdef USE_MIGEMO
+	if (kzembed->priv->migemo_keyword)
+		g_free(kzembed->priv->migemo_keyword);
+	kzembed->priv->migemo_keyword =NULL;
+#endif
 }
 
 
@@ -581,8 +585,16 @@ kz_moz_embed_location (GtkMozEmbed *embed)
 	KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);
 
 	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
-
-	g_free(kzembed->location);
+	
+	if (kzembed->priv->first_url)
+	{
+		kz_moz_embed_load_url(kzembed, kzembed->priv->first_url);
+		g_free(kzembed->priv->first_url);
+		kzembed->priv->first_url = NULL;
+		return;
+	}
+	if (kzembed->location)
+		g_free(kzembed->location);
 	kzembed->location = gtk_moz_embed_get_location(embed);
 }
 
@@ -600,6 +612,64 @@ kz_moz_embed_net_start (GtkMozEmbed *embed)
 	kzembed->is_loading = TRUE;
 }
 
+static void
+net_stop_proccess (KzMozEmbed *kzembed)
+{
+	gboolean create_thumbnail = FALSE, store_cache = TRUE;
+	KZ_CONF_GET("Global", "create_thumbnail", create_thumbnail, BOOL);
+	KZ_CONF_GET("History", "store_cache", store_cache, BOOL);
+
+	gchar *cache = g_strconcat("file://", g_get_home_dir(),
+			    	   HISTORY_DIR, NULL);
+	const gchar *location = kz_moz_embed_get_location(kzembed);
+	if (location && 
+	    (g_str_has_prefix(location, "http:") || 
+	     g_str_has_prefix(location, "https:") || 
+	     g_str_has_prefix(location, "history-search:") || 
+	     g_str_has_prefix(location, "file:")) &&
+	    !g_str_has_prefix(location, cache))
+	{
+		//get the last modification time
+		nsCOMPtr<nsIDOMDocument> domDoc;
+        	kzembed->priv->wrapper->GetMainDomDocument(getter_AddRefs(domDoc));
+		nsCOMPtr<nsIDOMNSDocument> doc = do_QueryInterface(domDoc);
+		nsAutoString value;
+		doc->GetLastModified(value);
+		nsTime lm (NS_ConvertUCS2toUTF8(value).get(), PR_TRUE);
+		GTime last_modified;
+		LL_DIV(last_modified,
+		       NS_STATIC_CAST(PRTime, lm), PR_USEC_PER_SEC);
+
+		nsCOMPtr<nsIURI> inURI;
+		nsCAutoString sURI;
+		nsresult rv;
+
+		rv = kzembed->priv->wrapper->GetDocumentUrl(sURI);
+      		rv = NS_NewURI(getter_AddRefs(inURI), sURI);
+
+		gchar *uri = ToNewCString(sURI);
+
+		if (create_thumbnail && 
+		    (!last_modified || (thumbnail_get_last_modified(uri) < last_modified)))
+		{
+			kz_moz_embed_create_thumbnail(kzembed);
+		}
+		
+		if (store_cache && exists_estindex &&
+		    (!last_modified || (history_get_last_modified(uri) < last_modified)) &&
+		    !g_str_has_prefix(location, "history-search:"))
+		{
+			gchar *filename;
+			filename = mozilla_store_history_file(kzembed);
+			if (filename)
+				g_idle_add(estsearch_update_index, filename);
+		}
+		g_free(uri);
+	}
+	
+	g_free(cache);
+}
+
 static void
 kz_moz_embed_net_stop (GtkMozEmbed *embed)
 {
@@ -616,6 +686,8 @@ kz_moz_embed_net_stop (GtkMozEmbed *embed)
 	kz_moz_embed_navigation_link_free(kzembed);
 
 	kz_moz_embed_get_head_link(kzembed);
+
+	net_stop_proccess(kzembed);
 }
 
 static void
@@ -711,6 +783,17 @@ kz_moz_embed_visibility (GtkMozEmbed *embed, gboolean visibility)
 	}
 }
 
+#if 0
+static void
+kz_moz_embed_security_change (GtkMozEmbed *embed,
+			      gpointer request,
+			      guint state)
+{
+	if (parent_class->security_change)
+		parent_class->security_change(embed, request, state);
+}
+#endif
+
 gboolean
 kz_moz_embed_can_cut_selection (KzMozEmbed *kzembed)
 {
@@ -813,6 +896,28 @@ kz_moz_embed_get_selection_string(KzMozEmbed *kzembed)
 	return g_strdup (NS_ConvertUCS2toUTF8(string).get());
 }
 
+gchar *
+kz_moz_embed_get_html_with_contents (KzMozEmbed *kzembed, const gchar *storedir)
+{
+	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), NULL);
+
+	if (!kzembed->priv->wrapper) return NULL;
+
+	nsresult rv;
+
+	nsCOMPtr<nsISelection> selection;
+	rv = kzembed->priv->wrapper->GetSelection(getter_AddRefs(selection));
+	if (!selection) return NULL;
+
+	nsAutoString string;
+	rv = kzembed->priv->wrapper->GetHtmlWithContents(selection, 
+							 storedir,
+							 string);
+	if (NS_FAILED(rv)) return NULL;
+
+	return g_strdup(NS_ConvertUCS2toUTF8(string).get());
+}
+#if 0
 gchar *
 kz_moz_embed_get_selection_source(KzMozEmbed *kzembed)
 {
@@ -832,6 +937,7 @@ kz_moz_embed_get_selection_source(KzMozEmbed *kzembed)
 
 	return g_strdup(NS_ConvertUCS2toUTF8(string).get());
 }
+#endif
 
 gboolean
 kz_moz_embed_find (KzMozEmbed *embed, const char *keyword,
@@ -848,13 +954,42 @@ kz_moz_embed_find (KzMozEmbed *embed, const char *keyword,
 	nsresult rv;
 	nsCOMPtr<nsIWebBrowserFind> finder(do_GetInterface(web));
 #if USE_MIGEMO
-	if (migemo && embed->priv->migemo_keyword)
-		finder->SetSearchString(NS_ConvertUTF8toUCS2(embed->priv->migemo_keyword).get());
+	gboolean use_migemo;
+	KZ_CONF_GET("Global", "use_migemo", use_migemo, BOOL);
+
+	if (use_migemo)
+	{
+		nsAutoString text;
+		gchar *body_string;
+
+		rv = embed->priv->wrapper->GetStringSelection(text, backward);
+		if (NS_FAILED(rv))
+			goto START_SEARCH;
+		body_string = g_strdup(NS_ConvertUCS2toUTF8(text).get());
+
+		if (embed->priv->migemo_keyword)
+			g_free(embed->priv->migemo_keyword);
+		embed->priv->migemo_keyword =
+			migemo_get_matched_text(body_string, keyword);
+		
+		if (embed->priv->migemo_keyword)
+		{
+			finder->SetSearchString(NS_ConvertUTF8toUCS2(embed->priv->migemo_keyword).get());
+		}
+		else
+		{
+			finder->SetSearchString(NS_ConvertUTF8toUCS2(keyword).get());
+		}
+		g_free(body_string);
+	}
 	else
+	{
 		finder->SetSearchString(NS_ConvertUTF8toUCS2(keyword).get());
+	}
 #else
 	finder->SetSearchString(NS_ConvertUTF8toUCS2(keyword).get());
 #endif
+START_SEARCH:
 	finder->SetFindBackwards(backward);
 	finder->SetWrapFind(TRUE);
 	finder->SetEntireWord(TRUE);
@@ -870,10 +1005,11 @@ gboolean
 kz_moz_embed_incremental_search (KzMozEmbed *embed, const char *keyword,
 				 gboolean backward)
 {
+	nsresult rv;
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(embed), FALSE);
 	g_return_val_if_fail(keyword, FALSE);
 	
-	if (strlen(keyword) ==0)
+	if (strlen(keyword) == 0)
 		return FALSE;
 
 	nsCOMPtr<nsIWebBrowser> web;
@@ -881,47 +1017,40 @@ kz_moz_embed_incremental_search (KzMozEmbed *embed, const char *keyword,
 					getter_AddRefs(web));
 	if (!web) return FALSE;
 
-	nsCOMPtr<nsISelection> selection;
-	nsresult rv;
-
-	rv = embed->priv->wrapper->GetSelection(getter_AddRefs(selection));
-	if (selection)
-	{
-		selection->RemoveAllRanges();
-	}
-
 	nsCOMPtr<nsIWebBrowserFind> finder(do_GetInterface(web));
 #if USE_MIGEMO
 	gboolean use_migemo;
 	KZ_CONF_GET("Global", "use_migemo", use_migemo, BOOL);
 
+	nsCOMPtr<nsISelection> selection;
+	rv = embed->priv->wrapper->GetSelection(getter_AddRefs(selection));
+	if (!selection) return FALSE;
+
+	selection->RemoveAllRanges();
+
 	if (use_migemo)
 	{
 		nsAutoString text;
 		gchar *body_string;
 
-		embed->priv->wrapper->GetBodyString(text);
+		rv = embed->priv->wrapper->GetBodyString(text);
+		if (NS_FAILED(rv))
+			goto START_SEARCH;
 		body_string = g_strdup(NS_ConvertUCS2toUTF8(text).get());
 
-		if (!migemo)
+		if (embed->priv->migemo_keyword)
+			g_free(embed->priv->migemo_keyword);
+		embed->priv->migemo_keyword =
+			migemo_get_matched_text(body_string, keyword);
+		
+		if (embed->priv->migemo_keyword) 
 		{
-			if (!migemo_init())
-			{
-				finder->SetSearchString(NS_ConvertUTF8toUCS2(keyword).get());
-				goto START_SEARCH;
-			}
-		}
-
-		gchar *word = migemo_get_word(body_string, keyword);
-
-		if (word) {
-			if (embed->priv->migemo_keyword)
-				g_free(embed->priv->migemo_keyword);
-
-			embed->priv->migemo_keyword = word;
-
 			finder->SetSearchString(NS_ConvertUTF8toUCS2(embed->priv->migemo_keyword).get());
 		}
+		else 
+		{
+			finder->SetSearchString(NS_ConvertUTF8toUCS2(keyword).get());
+		}
 		g_free(body_string);
 	}
 	else
@@ -979,51 +1108,21 @@ kz_moz_embed_get_links (KzMozEmbed *kzembed, GList **list,
 	if (NS_FAILED(rv)) return FALSE;
 
 	// get all anchor nodes in the document.
-	nsCOMPtr<nsIDOMDocument> domDoc;
-        rv = kzembed->priv->wrapper->GetMainDomDocument(getter_AddRefs(domDoc));
-        if (NS_FAILED(rv) || !domDoc) return FALSE;
+	nsCOMPtr<nsIDOMDocument> mainDoc;
+        rv = kzembed->priv->wrapper->GetMainDomDocument(getter_AddRefs(mainDoc));
+        if (NS_FAILED(rv) || !mainDoc) return FALSE;
 
-	nsAutoString tagname;
-	tagname.AssignWithConversion("a");
-	nsCOMPtr<nsIDOMNodeList> nodeList;
-	rv = domDoc->GetElementsByTagName(tagname,
-					  getter_AddRefs(nodeList));
-        if (NS_FAILED(rv) || !domDoc) return FALSE;
+        // get main DOMWindow
+	nsCOMPtr<nsIDOMWindow> mainDOMWindow;
+	rv = kzembed->priv->wrapper->GetDOMWindow(getter_AddRefs(mainDOMWindow));
+	if (NS_FAILED(rv)) return FALSE;
+				
+	rv = kzembed->priv->wrapper->GetLinksFromWindow(mainDOMWindow,
+							list,
+							selection,
+							selected_only);
 
-	PRUint32 num;
-	rv = nodeList->GetLength(&num);
-	if (NS_FAILED(rv) || num < 1) return FALSE;
-
-	// store links to GList
-	nsCOMPtr<nsIDOMNode> node;
-	for (PRUint32 i = 0; i < num; i++)
-	{
-		rv = nodeList->Item(i, getter_AddRefs(node));
-		if (NS_FAILED(rv) || !node) continue;
-
-		// check whether the selection contains these nodes or not.
-		if (selected_only)
-		{
-			PRBool contains;
-			selection->ContainsNode(node, PR_TRUE, &contains);
-			if (!contains) continue;
-		}
-
-		gchar *uri = NULL, *title = NULL;
-		mozilla_get_link_from_node(domDoc, node,
-					   &uri, &title);
-		if (uri && *uri)
-		{
-			KzBookmark *link;
-			link = kz_bookmark_new_with_attrs(title, uri, NULL);
-			*list = g_list_append(*list, link);
-		}
-
-		g_free(uri);
-		g_free(title);
-	}
-
-	return TRUE;
+	return NS_FAILED(rv) ? FALSE : TRUE;
 }
 
 
@@ -1089,8 +1188,8 @@ kz_moz_embed_get_key_event_info(KzMozEmbed *kzembed, gpointer event,
      	nsString nodename;
      	OriginalNode->GetNodeName(nodename);
 
-     	if (nodename.EqualsWithConversion("xul:thumb") ||
-	    nodename.EqualsWithConversion("xul:slider"))
+     	if (nodename.EqualsIgnoreCase("xul:thumb") ||
+	    nodename.EqualsIgnoreCase("xul:slider"))
 	{
        		return KZ_CONTEXT_NONE;
 	}
@@ -1154,8 +1253,8 @@ kz_moz_embed_get_mouse_event_info(KzMozEmbed *kzembed, gpointer event,
      	nsString nodename;
      	OriginalNode->GetNodeName(nodename);
 
-     	if (nodename.EqualsWithConversion ("xul:thumb") ||
-	    nodename.EqualsWithConversion ("xul:slider"))
+     	if (nodename.EqualsIgnoreCase("xul:thumb") ||
+	    nodename.EqualsIgnoreCase("xul:slider"))
 	{
        		return KZ_CONTEXT_NONE;
 	}
@@ -1171,9 +1270,11 @@ kz_moz_embed_get_mouse_event_info(KzMozEmbed *kzembed, gpointer event,
 static void
 kz_moz_embed_get_head_link (KzMozEmbed *kzembed)
 {
+	g_return_if_fail(kzembed->priv->wrapper);
 	nsresult rv;
 	nsCOMPtr<nsIDOMDocument> domDoc;
-        rv = kzembed->priv->wrapper->GetMainDomDocument(getter_AddRefs(domDoc));
+	KzMozWrapper *wrapper = kzembed->priv->wrapper;
+        rv = wrapper->GetMainDomDocument(getter_AddRefs(domDoc));
         if (NS_FAILED(rv) || !domDoc) return;
 
 	nsAutoString tagname;
@@ -1193,10 +1294,12 @@ kz_moz_embed_get_head_link (KzMozEmbed *kzembed)
 		rv = nodeList->Item(i, getter_AddRefs(node));
 		if (NS_FAILED(rv) || !node) continue;
 		
-		char *relattr =  mozilla_get_attribute(node, "rel");
+		char *relattr = NULL;
+		wrapper->GetAttributeFromNode(node, "rel", &relattr);
 		if (!relattr) continue;
 
-		mozilla_get_head_link_from_node(domDoc, node, &url);
+		wrapper->GetLinkFromNode(domDoc, node, &url);
+		if (!url) continue;
 
 		nsAutoString rela;
 		rela.AssignWithConversion(relattr);
@@ -1210,29 +1313,13 @@ kz_moz_embed_get_head_link (KzMozEmbed *kzembed)
 				      kz_moz_embed_signals[FAVICON_SIGNAL],
 				      0, url);
 		}
-		else if(!strcasecmp(rel, "ALTERNATE"))
+		for (int j = 0; j < N_LINKS; j++)
 		{
-			kzembed->priv->link_rss = g_strdup(url);
-		}
-		else if(!strcasecmp(rel, "INDEX"))
-		{
-			kzembed->priv->link_index = g_strdup(url);
-		} 
-		else if(!strcasecmp(rel, "CONTENTS"))
-		{
-			kzembed->priv->link_contents = g_strdup(url);
-		}
-		else if (!strcasecmp(rel, "PREV"))
-		{
-			kzembed->priv->link_prev = g_strdup(url);
-		}
-		else if (!strcasecmp(rel, "NEXT"))
-		{
-			kzembed->priv->link_next = g_strdup(url);
-		}
-		else if (!strcasecmp(rel, "START"))
-		{
-			kzembed->priv->link_start = g_strdup(url);
+			if (!strcasecmp(rel, nav_names[j]))
+			{
+				kzembed->priv->nav_links[j] = g_strdup(url);
+				break;
+			}
 		}
 		g_free(url);
 		g_free(rel);
@@ -1241,105 +1328,14 @@ kz_moz_embed_get_head_link (KzMozEmbed *kzembed)
 }
 
 
-// Nautilus CREDITS here
-static gchar *
-mozilla_get_attribute (nsIDOMNode *node, gchar *attribute)
-{
-     	nsresult result;
-
-	nsCOMPtr<nsIDOMNamedNodeMap> attributes;
-	result = node->GetAttributes(getter_AddRefs(attributes));
-	if (!NS_SUCCEEDED (result) || !attributes) return NULL;
-
-	nsAutoString attr; 
-
-	attr.AssignWithConversion(attribute);
-
-	nsCOMPtr<nsIDOMNode> attrNode;
-	result = attributes->GetNamedItem (attr, getter_AddRefs(attrNode));
-	if (!NS_SUCCEEDED(result) || !attrNode)  return NULL;
-
-	nsAutoString nodeValue;
-
-	result = attrNode->GetNodeValue(nodeValue);
-	if (!NS_SUCCEEDED(result))  return NULL;
-
-	return ToNewCString(nodeValue);
-}
-
-
-static gboolean
-mozilla_get_link_from_node (nsIDOMDocument *domDoc, nsIDOMNode *node,
-			    gchar **url, gchar **inner_html)
-{
-	if (inner_html) *inner_html = NULL;
-	if (url) *url = NULL;
-
-	// get url
-	char *hrefattr =  mozilla_get_attribute(node, "href");
-	if (!hrefattr) return FALSE;
-
-	nsAutoString hrefa;
-	hrefa.AssignWithConversion(hrefattr);
-
-	nsCString hrefc,linkc;
-	hrefc.AssignWithConversion(hrefa);
-
-	nsIURI *baseURI;
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
-	nsresult rv;
-#if MOZILLA_SNAPSHOT > 12
-	baseURI = doc->GetBaseURI();
-#elif MOZILLA_SNAPSHOT > 11
-	baseURI = doc->GetBaseURL();
-#elif MOZILLA_SNAPSHOT > 9
-	rv = doc->GetBaseURL(&baseURI);
-#else
-	rv = doc->GetBaseURL(baseURI);
-#endif	
-	rv = baseURI->Resolve(hrefc,linkc);
-	*url = ToNewCString(linkc);
-
-	g_free(hrefattr);
-
-	// get title
-	nsAutoString nodename;
-	node->GetNodeName(nodename);
-	nsCOMPtr<nsIDOMNamedNodeMap> attributes;
-	node->GetAttributes(getter_AddRefs(attributes));
-
-	nsCOMPtr<nsIDOMNode> hrefNode;
-	nsAutoString href; 
-	href.AssignWithConversion("href");
-	attributes->GetNamedItem(href, getter_AddRefs(hrefNode));
-	if (!hrefNode) return FALSE;
-
-	nsAutoString linkhtml;
-	nsCOMPtr<nsIDOMNSHTMLElement> nsElement;
-
-	nsCOMPtr<nsIDOMHTMLElement> element;
-	element = do_QueryInterface(node);
-
-	nsElement = do_QueryInterface(element);
-	if (!nsElement) return FALSE;
-
-	rv = nsElement->GetInnerHTML(linkhtml);
-	if (NS_SUCCEEDED(rv) &&
-	    NS_ConvertUCS2toUTF8(linkhtml).get()) 
-	{
-		*inner_html = g_strdup(NS_ConvertUCS2toUTF8(linkhtml).get());
-	}
-
-	return TRUE;
-}
-
-
 static glong
 mozilla_set_event_context (KzMozEmbed *kzembed,
 			   nsIDOMEventTarget *target,
 			   KzEmbedEvent *info)
 {
 	nsresult result;
+	KzMozWrapper *wrapper = kzembed->priv->wrapper;
+	g_return_val_if_fail(kzembed->priv->wrapper, KZ_CONTEXT_NONE);
 
 	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(target);
 	if (!node) return KZ_CONTEXT_NONE;
@@ -1368,7 +1364,7 @@ mozilla_set_event_context (KzMozEmbed *kzembed,
 
      	nsString mime;
      	nsDoc->GetContentType(mime);
-     	if (mime.EqualsWithConversion("text/xul"))  return KZ_CONTEXT_NONE;
+     	if (mime.EqualsIgnoreCase("text/xul"))  return KZ_CONTEXT_NONE;
 
 	PRUint32 flags = KZ_CONTEXT_NONE;
 
@@ -1405,22 +1401,25 @@ mozilla_set_event_context (KzMozEmbed *kzembed,
 		if (element)
 		{
 			nsAutoString tag;
+			gchar *utf8_tag;
 			element->GetTagName(tag);
+			utf8_tag = g_strdup(NS_ConvertUCS2toUTF8(tag).get());
 
-			if (tag.EqualsWithConversion("input", PR_TRUE))
+			if (!g_ascii_strcasecmp(utf8_tag, "input"))
 			{
 				flags |= KZ_CONTEXT_INPUT;
 			}
-			else if (tag.EqualsWithConversion("textarea", PR_TRUE))
+			else if (!g_ascii_strcasecmp(utf8_tag, "textarea"))
 			{
 				flags |= KZ_CONTEXT_INPUT | KZ_CONTEXT_TEXTAREA;
 				info->element = (void*)element;
 			}
-			else if (tag.EqualsWithConversion("img", PR_TRUE))
+			else if (!g_ascii_strcasecmp(utf8_tag, "img"))
 			{
 				flags |= KZ_CONTEXT_IMAGE;
 
-				char *src = mozilla_get_attribute(node, "src");
+				char *src = NULL;
+				wrapper->GetAttributeFromNode(node, "src", &src);
 				if (!src)  return KZ_CONTEXT_NONE;
 
 			     	nsAutoString srca;
@@ -1440,6 +1439,7 @@ mozilla_set_event_context (KzMozEmbed *kzembed,
 			{
 				flags |= KZ_CONTEXT_OTHER;
 			}
+			g_free(utf8_tag);
 
 			nsCOMPtr<nsIDOMNamedNodeMap> attributes;
 			node->GetAttributes(getter_AddRefs(attributes));
@@ -1453,9 +1453,10 @@ mozilla_set_event_context (KzMozEmbed *kzembed,
 				{
 					flags |= KZ_CONTEXT_LINK;
 
-					mozilla_get_link_from_node(domDoc, node,
-								   &info->link,
-								   &info->linktext);
+					wrapper->GetLinkAndTitleFromNode(domDoc,
+								 node,
+								 &info->link,
+								 &info->linktext);
 					if (!info->link)
 					{
 						g_free(info->linktext);
@@ -1483,44 +1484,6 @@ mozilla_set_event_context (KzMozEmbed *kzembed,
 	return flags;
 }
 
-
-static gboolean
-mozilla_get_head_link_from_node (nsIDOMDocument *domDoc, nsIDOMNode *node,
-				 gchar **url)
-{
-	if (url) *url = NULL;
-
-	// get url
-	char *hrefattr =  mozilla_get_attribute(node, "href");
-	if (!hrefattr)
-		return FALSE;	
-
-	
-	nsAutoString hrefa;
-	hrefa.AssignWithConversion(hrefattr);
-
-	nsCString hrefc,linkc;
-	hrefc.AssignWithConversion(hrefa);
-
-	nsIURI *baseURI;
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
-	nsresult rv;
-#if MOZILLA_SNAPSHOT > 12
-	baseURI = doc->GetBaseURI();
-#elif MOZILLA_SNAPSHOT > 11
-	baseURI = doc->GetBaseURL();
-#elif MOZILLA_SNAPSHOT > 9
-	rv = doc->GetBaseURL(&baseURI);
-#else
-	rv = doc->GetBaseURL(baseURI);
-#endif	
-	rv = baseURI->Resolve(hrefc,linkc);
-	*url = ToNewCString(linkc);
-
-	g_free(hrefattr);
-	return TRUE;
-}
-
 static gchar *
 kz_moz_embed_get_up_location(KzMozEmbed *kzembed)
 {
@@ -1627,100 +1590,35 @@ kz_moz_embed_can_go_up(KzMozEmbed *kzembed)
 }
 
 gboolean
-kz_moz_embed_can_go_index(KzMozEmbed *kzembed)
+kz_moz_embed_can_go_nav_link (KzMozEmbed *kzembed,
+			      KzMozEmbedNavLink link)
 {
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
-	if(kzembed->priv->link_index)
-		return TRUE;
-	return FALSE;
-}
-
-gboolean
-kz_moz_embed_can_go_contents(KzMozEmbed *kzembed)
-{
-	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
-	if(kzembed->priv->link_contents)
-		return TRUE;
-	return FALSE;
-}
-
-gboolean
-kz_moz_embed_can_go_start(KzMozEmbed *kzembed)
-{
-	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
-	if(kzembed->priv->link_start)
-		return TRUE;
-	return FALSE;
-}
-gboolean
-kz_moz_embed_can_go_prev(KzMozEmbed *kzembed)
-{
-	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
-	if(kzembed->priv->link_prev)
-		return TRUE;
-	return FALSE;
-}
-
-gboolean
-kz_moz_embed_can_go_next(KzMozEmbed *kzembed)
-{
-	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
-	if(kzembed->priv->link_next)
-		return TRUE;
-	return FALSE;
+	return (kzembed->priv->nav_links[link]) ? TRUE : FALSE;
 }
 
 void
-kz_moz_embed_go_index (KzMozEmbed *kzembed)
+kz_moz_embed_go_nav_link (KzMozEmbed *kzembed,
+			  KzMozEmbedNavLink link)
 {
 	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
-	g_return_if_fail(kzembed->priv->link_index);
+	g_return_if_fail(kzembed->priv->nav_links[link]);
 
 	kz_moz_embed_load_url(kzembed,
-			      kzembed->priv->link_index);
+			      kzembed->priv->nav_links[link]);
 }
 
 void
-kz_moz_embed_go_contents (KzMozEmbed *kzembed)
+kz_moz_embed_go_history_index (KzMozEmbed *kzembed, gint index)
 {
 	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
-	g_return_if_fail(kzembed->priv->link_contents);
-
-	kz_moz_embed_load_url(kzembed,
-			      kzembed->priv->link_contents);
+	
+	KzMozWrapper *wrapper = kzembed->priv->wrapper;
+	g_return_if_fail(kzembed->priv->wrapper);
+	
+	wrapper->GoHistoryIndex(index);
 }
 
-void
-kz_moz_embed_go_start (KzMozEmbed *kzembed)
-{
-	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
-	g_return_if_fail(kzembed->priv->link_start);
-
-	kz_moz_embed_load_url(kzembed,
-			      kzembed->priv->link_start);
-}
-
-void
-kz_moz_embed_go_next (KzMozEmbed *kzembed)
-{
-	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
-	g_return_if_fail(kzembed->priv->link_next);
-
-	kz_moz_embed_load_url(kzembed,
-			      kzembed->priv->link_next);
-}
-
-void
-kz_moz_embed_go_prev (KzMozEmbed *kzembed)
-{
-	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
-	g_return_if_fail(kzembed->priv->link_prev);
-
-	kz_moz_embed_load_url(kzembed,
-			      kzembed->priv->link_prev);
-}
-
-
 void
 kz_moz_embed_copy_page (KzMozEmbed   *kzembed, KzMozEmbed   *dkzembed)
 {
@@ -1733,7 +1631,8 @@ kz_moz_embed_copy_page (KzMozEmbed   *kzembed, KzMozEmbed   *dkzembed)
         rv = sWrapper->GetPageDescriptor(getter_AddRefs(pageDescriptor));
         if (!pageDescriptor || NS_FAILED(rv)) return;
 
-        rv = dWrapper->LoadDocument(pageDescriptor, 0);
+        rv = dWrapper->LoadDocument(pageDescriptor, 
+				    nsIWebPageDescriptor::DISPLAY_NORMAL);
 }
 
 
@@ -1951,9 +1850,9 @@ kz_moz_embed_get_text_from_textarea (KzMozEmbed *kzembed,
 {
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), NULL);
 	
-	nsISupports *s = (nsISupports*)element;
+	nsIDOMHTMLElement *htmlElement = (nsIDOMHTMLElement*)element;
 
-	nsCOMPtr<nsIDOMHTMLTextAreaElement> tElement = do_QueryInterface(s);
+	nsCOMPtr<nsIDOMHTMLTextAreaElement> tElement = do_QueryInterface(htmlElement);
 
 	g_return_val_if_fail(tElement, NULL);
 
@@ -1971,9 +1870,10 @@ kz_moz_embed_set_text_into_textarea (KzMozEmbed *kzembed,
 {
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
 
-	nsISupports *s = (nsISupports*)element;
 
-	nsCOMPtr<nsIDOMHTMLTextAreaElement> tElement = do_QueryInterface(s);
+	nsIDOMHTMLElement *htmlElement = (nsIDOMHTMLElement*)element;
+
+	nsCOMPtr<nsIDOMHTMLTextAreaElement> tElement = do_QueryInterface(htmlElement);
 
 	if (!tElement) return FALSE;
 
@@ -1984,8 +1884,8 @@ kz_moz_embed_set_text_into_textarea (KzMozEmbed *kzembed,
 }
 
 
-gchar *
-kz_moz_embed_store_history_file(KzMozEmbed *kzembed)
+static gchar *
+mozilla_store_history_file(KzMozEmbed *kzembed)
 {
 	nsresult rv;
 	KzMozWrapper *wrapper = NULL;
@@ -2016,11 +1916,8 @@ kz_moz_embed_store_history_file(KzMozEmbed *kzembed)
 
 	filename = create_filename_with_path_from_uri(uri);
 		
-	if (!filename)
-		filename = g_strdup("index.html");
-
 	dir = g_strconcat(g_get_home_dir(),
-			  "/.kazehakase/history/",
+			  HISTORY_DIR,
 			  filename,
 			  NULL);
 	g_free(filename);
@@ -2051,3 +1948,106 @@ kz_moz_embed_store_history_file(KzMozEmbed *kzembed)
 	return dir;
 }
 
+void
+kz_moz_embed_create_thumbnail (KzMozEmbed *kzembed)
+{
+	KzMozWrapper *wrapper = NULL;
+	wrapper = kzembed->priv->wrapper;
+	g_return_if_fail (wrapper != NULL);
+
+	nsresult rv;
+	int total, index;
+	rv = wrapper->GetSHInfo (&total, &index);
+
+	nsCOMPtr<nsIHistoryEntry> he;
+	rv = wrapper->GetHistoryEntry(index, getter_AddRefs(he));
+	if (NS_FAILED(rv)) return;
+
+	GtkWidget *window = gtk_widget_get_toplevel(GTK_WIDGET(kzembed));
+
+	if (!KZ_IS_WINDOW(window)) return;
+
+	KzWindow *kz = KZ_WINDOW(window);
+
+	if (!kz->thumb_creator)
+		kz_window_create_thumbnail_creator(kz);
+
+	kz_moz_thumbnail_creator_append_queue(kz->thumb_creator,
+					      he);
+}
+
+
+void
+kz_moz_embed_do_command (KzMozEmbed *kzembed,
+			 const char *command)
+{
+	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
+	nsCOMPtr<nsICommandManager> commandManager;
+	commandManager = do_GetInterface(kzembed->priv->wrapper->mWebBrowser);
+	if (!commandManager) return;
+	
+	commandManager->DoCommand(command, nsnull, nsnull);
+}
+
+
+gboolean
+kz_moz_embed_can_do_command (KzMozEmbed *kzembed,
+			     const char *command)
+{
+	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
+	
+	nsCOMPtr<nsICommandManager> commandManager;
+	commandManager = do_GetInterface(kzembed->priv->wrapper->mWebBrowser);
+	if (!commandManager) return FALSE;
+	
+	PRBool enabled;
+	commandManager->IsCommandEnabled(command, nsnull, &enabled);
+
+	return (enabled == PR_TRUE);
+}
+
+void
+kz_moz_embed_zoom_set (KzMozEmbed *kzembed, 
+               	       int zoom, 
+               	       gboolean reflow)
+{
+	KzMozWrapper *wrapper = kzembed->priv->wrapper;
+
+	wrapper->SetImageZoom ((float)(zoom) / 100);
+		
+	kz_moz_embed_set_text_size(kzembed, zoom, reflow);
+}
+
+int 
+kz_moz_embed_zoom_get (KzMozEmbed *kzembed)
+{
+	return kz_moz_embed_get_text_size(kzembed);
+}
+
+
+void
+kz_moz_embed_set_text_size (KzMozEmbed *kzembed, 
+               	            int zoom, 
+               	            gboolean reflow)
+{
+	KzMozWrapper *wrapper = kzembed->priv->wrapper;
+
+	wrapper->SetZoom ((float)(zoom) / 100, reflow);
+}
+
+int 
+kz_moz_embed_get_text_size (KzMozEmbed *kzembed)
+{
+	KzMozWrapper *wrapper = kzembed->priv->wrapper;
+	float f;
+	int zoom;
+
+	nsresult result = wrapper->GetZoom (&f);
+	if (NS_FAILED (result)) return 100;
+
+	zoom = (int) rint (f * 100);
+
+	return zoom;
+}
+
+
