@@ -109,10 +109,18 @@ NS_IMETHODIMP KzContentHandler::PromptForSaveToFile(
 
 	const nsAString &title = NS_ConvertUTF8toUCS2(_("Select the destination filename"));
 
+#if MOZILLA_SNAPSHOT > 15
+	filePicker->Init(windowInternal,
+			 title, 
+			 nsIFilePicker::modeSave);
+	nsAutoString defaultFile(aDefaultFile);
+	filePicker->SetDefaultString(defaultFile);
+#else
 	filePicker->Init(windowInternal,
 			 PromiseFlatString(title).get(), 
 			 nsIFilePicker::modeSave);
 	filePicker->SetDefaultString(aDefaultFile);
+#endif
 	filePicker->SetDisplayDirectory(saveDir);
 
 	filePicker->Show(&okToSave);
@@ -153,7 +161,9 @@ NS_METHOD KzContentHandler::Init (void)
 
 	nsCOMPtr<nsIMIMEInfo> MIMEInfo;
 	rv = mLauncher->GetMIMEInfo(getter_AddRefs(MIMEInfo));
+#if MOZILLA_SNAPSHOT < 15
 	rv = MIMEInfo->GetMIMEType(&mMimeType);
+#endif
 
 #if MOZILLA_SNAPSHOT > 11
 	rv = mLauncher->GetSource(getter_AddRefs(mUri));
