@@ -597,6 +597,10 @@ kz_http_start (KzIO *io)
 	http = KZ_HTTP(io);
 	
 	/* proxy check */
+	KZ_CONF_GET("Global", "use_proxy", use_proxy, BOOL);
+	if (!use_proxy)
+		goto NO_PROXY;
+	
 	exist = KZ_CONF_GET("Global", "proxy_name", proxy_name, STRING);
 	if (!exist)
 	{
@@ -611,24 +615,13 @@ kz_http_start (KzIO *io)
 		   so connect without proxy */
 		goto NO_PROXY;
 	}
-	
+
+	exist = KZ_CONF_GET("Global", "proxy_name", proxy_name, STRING);
 	g_object_get(G_OBJECT(item),
-		     "use_proxy", &use_proxy,
 		     "http_host", &http_host,
 		     "http_port", &http_port,
 		     NULL);
 			
-	if (!use_proxy)
-	{
-		/* This proxy can't be used, so free item and http_host, and then 
-		   connect without proxy */
-		if(http_host)
-			g_free(http_host);
-
-		g_object_unref(G_OBJECT(item));
-		goto NO_PROXY;
-	}
-	
 	/* There is a valid proxy setting, so connect with this proxy*/
 	http->priv->use_proxy = TRUE;
 	gnet_tcp_socket_connect_async(http_host, http_port,
