@@ -52,9 +52,11 @@ typedef struct _KzPrefsGeneral
 	GtkWidget     *override_check;
 	GtkWidget     *user_agent_entry;
 	GtkWidget     *keep_bookmark_menu;
-	GtkWidget     *keep_bookmark_menu_frame;
-	GtkWidget     *keep_bookmark_menu_vbox;
+	GtkWidget     *confirm_open_bookmarks;
+	GtkWidget     *bookmark_menu_frame;
+	GtkWidget     *bookmark_menu_vbox;
 	GtkWidget     *open_in_new_tab;
+	GtkWidget     *create_thumbnail;
 	GtkOptionMenu *ui_menu;
 	gboolean       changed;
 } KzPrefsGeneral;
@@ -146,7 +148,8 @@ prefs_general_create (void)
 	GtkWidget *label, *entry, *button;
 	GtkWidget *option_menu, *menu, *check;
 	gchar *value;
-	gboolean keep = FALSE, newtab = FALSE;
+	gboolean keep = FALSE, newtab = FALSE, confirm = TRUE;
+	gboolean thumbnail = FALSE;
 #if 0
 	GtkWidget *check;
 	gboolean override = FALSE;
@@ -292,19 +295,19 @@ prefs_general_create (void)
 	 *  Bookmark menu
 	 */
 	frame = gtk_frame_new(_("Bookmark menu"));
-	prefsui->keep_bookmark_menu_frame = frame;
+	prefsui->bookmark_menu_frame = frame;
 	gtk_frame_set_label_align(GTK_FRAME(frame), 0.03, 0.5);
 	gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
 	gtk_box_pack_start(GTK_BOX(main_vbox), frame, FALSE, FALSE, 2);
 	gtk_widget_show(frame);
 
 	vbox = gtk_vbox_new(FALSE, 0);
-	prefsui->keep_bookmark_menu_vbox = vbox;
+	prefsui->bookmark_menu_vbox = vbox;
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 	gtk_widget_show(vbox);
 
-	check = gtk_check_button_new_with_mnemonic(_("Keep _popuping"));
+	check = gtk_check_button_new_with_mnemonic(_("Keep menu _open by middle click"));
 	prefsui->keep_bookmark_menu = check;
 	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
 	KZ_CONF_GET("Global", "keep_bookmark_menu", keep, BOOL);
@@ -313,10 +316,19 @@ prefs_general_create (void)
 			 G_CALLBACK(cb_changed), prefsui);
 	gtk_widget_show(check);
 
+	check = gtk_check_button_new_with_mnemonic(_("_Confirmation of opening all bookmarks in bookmark folder"));
+	prefsui->confirm_open_bookmarks = check;
+	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
+	KZ_CONF_GET("Global", "confirm_open_bookmarks", confirm, BOOL);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), confirm);
+	g_signal_connect(G_OBJECT(check), "toggled",
+			 G_CALLBACK(cb_changed), prefsui);
+	gtk_widget_show(check);
+
 	/*
 	 *  Entry Box behaviour
 	 */
-	frame = gtk_frame_new(_("Location entry and google entry"));
+	frame = gtk_frame_new(_("Location entry and search entry"));
 	gtk_frame_set_label_align(GTK_FRAME(frame), 0.03, 0.5);
 	gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
 	gtk_box_pack_start(GTK_BOX(main_vbox), frame, FALSE, FALSE, 2);
@@ -336,6 +348,29 @@ prefs_general_create (void)
 			 G_CALLBACK(cb_changed), prefsui);
 	gtk_widget_show(check);
 
+	/*
+	 *  Thumbnail 
+	 */
+	frame = gtk_frame_new(_("Thumbnail"));
+	gtk_frame_set_label_align(GTK_FRAME(frame), 0.03, 0.5);
+	gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
+	gtk_box_pack_start(GTK_BOX(main_vbox), frame, FALSE, FALSE, 2);
+	gtk_widget_show(frame);
+
+	vbox = gtk_vbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
+	gtk_container_add(GTK_CONTAINER(frame), vbox);
+	gtk_widget_show(vbox);
+
+	check = gtk_check_button_new_with_mnemonic(_("Create thumbnail"));
+	prefsui->create_thumbnail = check;
+	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
+	KZ_CONF_GET("Global", "create_thumbnail", thumbnail, BOOL);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), thumbnail);
+	g_signal_connect(G_OBJECT(check), "toggled",
+			 G_CALLBACK(cb_changed), prefsui);
+	gtk_widget_show(check);
+
 	/*
 	 *  UI Level
 	 */
@@ -388,7 +423,7 @@ prefs_general_response (GtkWidget *widget, gint response)
 	case GTK_RESPONSE_APPLY:
 	{
 		const gchar *value;
-		gboolean keep, newtab;
+		gboolean keep, newtab, thumbnail;
 #if 0
 		gboolean override;
 #endif
@@ -396,7 +431,7 @@ prefs_general_response (GtkWidget *widget, gint response)
 
 		/* statrup */
 		value = gtk_entry_get_text(GTK_ENTRY(prefsui->start_page_entry));
-		KZ_CONF_SET("Global", "startup_page", value, STRING);
+		KZ_CONF_SET_STR("Global", "startup_page", value);
 
 #if 0
 		/* user agent */
@@ -420,18 +455,27 @@ prefs_general_response (GtkWidget *widget, gint response)
 			(GTK_TOGGLE_BUTTON(prefsui->keep_bookmark_menu));
 		KZ_CONF_SET("Global", "keep_bookmark_menu", keep, BOOL);
 
+		keep = gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON(prefsui->confirm_open_bookmarks));
+		KZ_CONF_SET("Global", "confirm_open_bookmarks", keep, BOOL);
+		
 		/* entry box behaviour */
 		newtab = gtk_toggle_button_get_active
 			(GTK_TOGGLE_BUTTON(prefsui->open_in_new_tab));
 		KZ_CONF_SET("Global", "entry_open_in_new_tab", newtab, BOOL);
 
+		/* thumbnail */
+		thumbnail = gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON(prefsui->create_thumbnail));
+		KZ_CONF_SET("Global", "create_thumbnail", thumbnail, BOOL);
+
 		/* UI level */
 		if (prefsui->changed)
 		{
 			i = gtk_option_menu_get_history(prefsui->ui_menu);
 			if (i >= 0 && i < n_ui_levels)
-				KZ_CONF_SET("Global", "ui_level",
-					    ui_levels[i].code, STRING);
+				KZ_CONF_SET_STR("Global", "ui_level",
+						ui_levels[i].code);
 		}
 
 		prefsui->changed = FALSE;
@@ -449,8 +493,8 @@ prefs_general_response (GtkWidget *widget, gint response)
 		g_free(value);
 
 		/* Bookmark menu */
-		gtk_widget_hide(prefsui->keep_bookmark_menu_frame);
-		gtk_widget_hide(prefsui->keep_bookmark_menu_vbox);
+		gtk_widget_hide(prefsui->bookmark_menu_frame);
+		gtk_widget_hide(prefsui->bookmark_menu_vbox);
 		break;
 	}
 	case KZ_RESPONSE_UI_LEVEL_MEDIUM:
@@ -462,8 +506,8 @@ prefs_general_response (GtkWidget *widget, gint response)
 		g_free(value);	
 
 		/* Bookmark menu */
-		gtk_widget_hide(prefsui->keep_bookmark_menu_frame);
-		gtk_widget_hide(prefsui->keep_bookmark_menu_vbox);
+		gtk_widget_hide(prefsui->bookmark_menu_frame);
+		gtk_widget_hide(prefsui->bookmark_menu_vbox);
 		break;
 	}
 	case KZ_RESPONSE_UI_LEVEL_EXPERT:
@@ -473,10 +517,10 @@ prefs_general_response (GtkWidget *widget, gint response)
 		GTKUTIL_SET_DEFAULT_HISTORY(prefsui->ui_menu, value,
 					    ui_levels, n_ui_levels, code);
 		g_free(value);
-		
+	
 		/* Bookmark menu */
-		gtk_widget_show(prefsui->keep_bookmark_menu_frame);
-		gtk_widget_show(prefsui->keep_bookmark_menu_vbox);
+		gtk_widget_show(prefsui->bookmark_menu_frame);
+		gtk_widget_show(prefsui->bookmark_menu_vbox);
 		break;
 	}
 	case KZ_RESPONSE_UI_LEVEL_CUSTOM:
