@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: prefs_key_accel.c,v 1.8 2004/07/14 03:43:37 ikezoe Exp $
+ *  $Id: prefs_key_accel.c,v 1.10 2004/09/07 10:55:04 ikezoe Exp $
  */
 
 /*
@@ -30,7 +30,8 @@
 #include <string.h>
 
 #include "kazehakase.h"
-#include "utils/intl.h"
+#include "intl.h"
+#include "glib-utils.h"
 
 
 #define DATA_KEY "KzPrefsKeyAccel::info"
@@ -129,7 +130,7 @@ refresh_selected_row (KzPrefsKeyAccel *self)
 	GtkTreeView *treeview;
 	GtkTreeSelection *selection;
 	GtkTreeIter iter;
-	char *accel_path;
+	char *accel_path, *orig_accel_name;
 	GtkAccelKey key;
 
 	g_return_if_fail(self);
@@ -141,6 +142,7 @@ refresh_selected_row (KzPrefsKeyAccel *self)
 	/* get the accel path for the selected row */
 	gtk_tree_model_get(GTK_TREE_MODEL(self->accel_store), &iter,
 			   COLUMN_PATH, &accel_path,
+			   COLUMN_ACCEL, &orig_accel_name,
 			   TERMINATOR);
 	if (!accel_path) return;
 
@@ -150,13 +152,18 @@ refresh_selected_row (KzPrefsKeyAccel *self)
 
 		accel_name = gtk_accelerator_name(key.accel_key,
 						  key.accel_mods);
-		gtk_list_store_set(self->accel_store, &iter,
-				   COLUMN_ACCEL, accel_name,
-				   TERMINATOR);
+		if (g_strcmp(accel_name, orig_accel_name))
+		{
+			gtk_list_store_set(self->accel_store, &iter,
+					   COLUMN_ACCEL, accel_name,
+					   TERMINATOR);
+		}
 		g_free(accel_name);
 	}
 
 	g_free(accel_path);
+	if (orig_accel_name)
+		g_free(orig_accel_name);
 }
 
 
