@@ -204,17 +204,17 @@ prefs_privacy_create (void)
 	 */ 
 	prefsui->cookie_lifetime = radio1
 		= gtk_radio_button_new_with_mnemonic(NULL,
-						  _("Accept cookie normally"));
+						  _("Accept cookie _normally"));
 	gtk_box_pack_start(GTK_BOX(vbox), radio1, TRUE, TRUE, 2);
 
 	radio2 = gtk_radio_button_new_with_mnemonic_from_widget(
 		 	GTK_RADIO_BUTTON(radio1),
-			_("Accept for current session only"));
+			_("Accept for current _session only"));
 	gtk_box_pack_start(GTK_BOX(vbox), radio2, TRUE, TRUE, 2);
 
 	radio3 = gtk_radio_button_new_with_mnemonic_from_widget(
 		 	GTK_RADIO_BUTTON(radio1),
-			_("Accept cookie for"));
+			_("Accept cookie _for"));
 	hbox = gtk_hbox_new(FALSE, 0);	
 	gtk_box_pack_start(GTK_BOX(hbox), radio3, FALSE, FALSE, 0);
 	prefsui->span_spin = spin
@@ -226,13 +226,14 @@ prefs_privacy_create (void)
 	gtk_widget_set_sensitive(spin, FALSE);
 	g_signal_connect(G_OBJECT(spin), "value-changed",
 			 G_CALLBACK(cb_changed), prefsui);
-	label = gtk_label_new(_("days"));
+	label = gtk_label_new_with_mnemonic(_("_days"));
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), spin);
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);	
 	gtk_box_pack_start(GTK_BOX(vbox), hbox,   TRUE, TRUE, 2);
 
-	radio4 = gtk_radio_button_new_with_label_from_widget(
+	radio4 = gtk_radio_button_new_with_mnemonic_from_widget(
 		 	GTK_RADIO_BUTTON(radio1),
-			_("Ask for each cookie"));
+			_("Ask for _each cookie"));
 	hbox = gtk_hbox_new(FALSE, 0);	
 	gtk_box_pack_start(GTK_BOX(hbox), radio4, FALSE, FALSE, 0);
 	prefsui->except_check = check =
@@ -294,7 +295,7 @@ prefs_privacy_create (void)
 	gtk_widget_show(hbox);
 
 	/* remove button */
-	button = gtk_button_new_with_mnemonic(_("_Manage Stored Passwords"));
+	button = gtk_button_new_with_mnemonic(_("_Manage Stored Passwords..."));
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(cb_password_button_clicked), prefsui);
