@@ -28,7 +28,8 @@
 #include "intl.h"
 
 
-#define INNER_BORDER     2
+#define INNER_BORDER    2 
+#define ARROW_WIDTH     6 
 
 typedef enum {
 	CURSOR_STANDARD,
@@ -162,6 +163,7 @@ kz_entry_init (KzEntry *entry)
 	entry->icon_type    = KZ_ENTRY_ICON_EMPTY;
 	entry->icon_width   = 0;
 	entry->icon_height  = 0;
+	entry->with_arrow   = FALSE;
 
 	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, 
 			     &entry->icon_width,
@@ -253,6 +255,7 @@ kz_entry_realize (GtkWidget *widget)
 
 	attributes.window_type = GDK_WINDOW_CHILD;
   
+	/* create the window for back ground */
 	get_widget_window_size (entry, &attributes.x, &attributes.y, &attributes.width, &attributes.height);
 
 	attributes.wclass = GDK_INPUT_OUTPUT;
@@ -273,6 +276,7 @@ kz_entry_realize (GtkWidget *widget)
 	widget->window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attributes_mask);
 	gdk_window_set_user_data (widget->window, entry);
 
+	/* create the window for text area */
 	get_text_area_size (entry, &attributes.x, &attributes.y, &attributes.width, &attributes.height);
 
 	attributes.cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget), GDK_XTERM);
@@ -283,6 +287,9 @@ kz_entry_realize (GtkWidget *widget)
 	entry->text_area = gdk_window_new (widget->window, &attributes, attributes_mask);
 	gdk_window_set_user_data (entry->text_area, entry);
 
+	gdk_cursor_unref (attributes.cursor);
+
+	/* create the window for icon area */
 	attributes.x -= KZ_ENTRY(entry)->icon_width + INNER_BORDER;
 	attributes.width = KZ_ENTRY(entry)->icon_width + INNER_BORDER;
 	attributes.cursor = gdk_cursor_new(GDK_LEFT_PTR);
@@ -291,6 +298,7 @@ kz_entry_realize (GtkWidget *widget)
 
 	gdk_cursor_unref (attributes.cursor);
 
+	/* set some properties */
 	widget->style = gtk_style_attach (widget->style, widget->window);
 
 	gdk_window_set_background (widget->window, &widget->style->base[GTK_WIDGET_STATE (widget)]);
@@ -1249,6 +1257,7 @@ kz_entry_expose (GtkWidget *widget,
 	{
 		GdkGC *gc;
 		gint x, y; 
+		GdkRectangle rect;
 		GdkPixbuf *scale_pixbuf;
 		get_text_area_size (GTK_ENTRY(entry),
 				    NULL, NULL,
@@ -1271,6 +1280,22 @@ kz_entry_expose (GtkWidget *widget,
 					-1, -1,
 					GDK_RGB_DITHER_NONE,
 					0, 0);
+			if (KZ_ENTRY(entry)->with_arrow)
+			{
+			rect.x = rect.y = 0;
+			rect.width = rect.height = x + ARROW_WIDTH;
+			gtk_paint_arrow(widget->style,
+					KZ_ENTRY(entry)->icon_area,
+					GTK_STATE_NORMAL,
+					GTK_SHADOW_NONE,
+					&rect,
+					widget,
+					NULL,
+					GTK_ARROW_DOWN,
+					TRUE,
+					x, y,
+					ARROW_WIDTH, ARROW_WIDTH);
+			}
 			g_object_unref(scale_pixbuf);
 			g_object_unref(gc);
 			break;
@@ -1300,6 +1325,22 @@ kz_entry_expose (GtkWidget *widget,
 					-1, -1,
 					GDK_RGB_DITHER_NONE,
 					0, 0);
+			if (KZ_ENTRY(entry)->with_arrow)
+			{
+			rect.x = rect.y = 0;
+			rect.width = rect.height = x + ARROW_WIDTH;
+			gtk_paint_arrow(widget->style,
+					KZ_ENTRY(entry)->icon_area,
+					GTK_STATE_NORMAL,
+					GTK_SHADOW_NONE,
+					&rect,
+					widget,
+					NULL,
+					GTK_ARROW_DOWN,
+					TRUE,
+					x, y,
+					ARROW_WIDTH, ARROW_WIDTH);
+			}
 			g_object_unref(gc);
 			g_object_unref(pixbuf);
 			g_object_unref(scale_pixbuf);
@@ -1542,6 +1583,16 @@ kz_entry_set_backtext (KzEntry *entry,
 }
 
 
+void
+kz_entry_set_arrow (KzEntry  *entry,
+		    gboolean arrow)
+{
+	g_return_if_fail (KZ_IS_ENTRY (entry));
+
+	entry->with_arrow = arrow;
+}
+
+
 void
 kz_entry_set_icon_from_stock (KzEntry  *entry,
 			      const gchar *stock_id,
@@ -1560,7 +1611,9 @@ kz_entry_set_icon_from_stock (KzEntry  *entry,
 	gtk_icon_size_lookup(size, 
 			     &entry->icon_width,
 			     &entry->icon_height);
-	
+	if (entry->with_arrow)
+		entry->icon_width += ARROW_WIDTH;
+
 	g_object_notify(G_OBJECT (entry), "stock");
   
 	g_object_thaw_notify (G_OBJECT (entry));
@@ -1595,7 +1648,8 @@ kz_entry_set_icon_from_pixbuf (KzEntry *entry,
 	}
 	entry->icon_width  = gdk_pixbuf_get_width(entry->pixbuf);
 	entry->icon_height = gdk_pixbuf_get_height(entry->pixbuf);
-
+	if (entry->with_arrow)
+		entry->icon_width += ARROW_WIDTH;
 	g_object_notify(G_OBJECT (entry), "pixbuf");
   
 	g_object_thaw_notify (G_OBJECT (entry));
